

from .. import skiboot, tag
from . import Widget, FieldArg, FieldArgList, FieldArgTable, FieldArgDict


class ColorTable1(Widget):
    """A general table of coloured text strings
       A list of header strings, defines the number of columns
       A list of lists of text, text colour, background colour
       Note : there is no error display"""

    # This class does not display any error messages
    display_errors = False

    arg_descriptions = {'header_class':FieldArg("cssclass",""),
                        'even_class':FieldArg("cssclass", ""),
                        'odd_class':FieldArg("cssclass", ""),
                        'titles': FieldArgList('text', valdt=False, jsonset=False),
                        'contents': FieldArgTable(['text', 'text', 'text'])
                        }

    def __init__(self, name=None, brief='', **field_args):
        """
        header_class: class of the header row, if empty string, then no class will be applied
        even_class: class of even rows, if empty string, then no class will be applied
        odd_class: class of odd rows, if empty string, then no class will be applied
        titles: List of text used as headers, gives the number of columns
        contents: list of lists, 0:text in the table, 1:the text color, 2:the background color
        """
        Widget.__init__(self, name=name, tag_name="table", brief=brief, **field_args)

    def _build(self, page, ident_list, environ, call_data, lang):
        "Build the table"
        titles = self.get_field_value("titles")
        contents = self.get_field_value("contents")
        cols = len(titles)
        if (not contents) or (not cols):
            return
        rows = int(len(contents) / cols)
        if not rows:
            return

        # set even row class
        if self.get_field_value('even_class'):
            even = self.get_field_value('even_class')
        else:
            even = ''
        # set odd row class
        if self.get_field_value('odd_class'):
            odd = self.get_field_value('odd_class')
        else:
            odd = ''

        if self.get_field_value('header_class'):
            self[0] = tag.Part(tag_name='tr', attribs={"class":self.get_field_value('header_class')})
        else:
            self[0] = tag.Part(tag_name='tr')

        for col in range(cols):
            self[0][col] = tag.Part(tag_name='th', text = titles[col])

        index = 0

        for row in range(rows):
            rownumber = row +1
            if even and (rownumber % 2):
                self[rownumber] = tag.Part(tag_name="tr", attribs={"class":even})
            elif odd and not (rownumber % 2):
                self[rownumber] = tag.Part(tag_name='tr', attribs={"class":odd})
            else:
                self[rownumber] = tag.Part(tag_name='tr')
            for col in range(cols):
                content = contents[index]
                celltext, cellcolor, cellbackground = contents[index]
                index += 1
                if cellcolor and cellbackground:
                    self[rownumber][col] = tag.Part(tag_name='td', text = celltext, attribs = {'style':'color:%s;background-color:%s;' % (cellcolor, cellbackground)})
                elif cellcolor:
                    self[rownumber][col] = tag.Part(tag_name='td', text = celltext, attribs = {'style':'color:%s;' % (cellcolor,)})
                elif cellbackground:
                    self[rownumber][col] = tag.Part(tag_name='td', text = celltext, attribs = {'style':'background-color:%s;' % (cellbackground,)})
                else:
                    self[rownumber][col] = tag.Part(tag_name='td', text = celltext)


    @classmethod
    def description(cls):
        """Returns a text string to illustrate the widget"""
        return """
<table>  <!-- with widget id and class widget_class -->
  <tr> <!-- with header class -->
    <th> <!-- title taken from titles list --> </th>
    <!-- continued with number of titles in titles list -->
  </tr>
  <tr> <!-- with class  from even or odd classes -->
    <td> <!-- with style from each contents element index 1 and 2 -->
      <!-- with text from each contents element index 0 -->
    </td>
    <!-- with appropriate number of columns matching titles-->
  </tr>
  <!-- with appropriate number of rows -->
</table>"""


class TwoColTable1(Widget):
    """A table of two columns, the columns being text strings
       The first row is two header titles
       Note : there is no error display"""

    # This class does not display any error messages
    display_errors = False

    arg_descriptions = {'header_class':FieldArg("cssclass",""),
                        'even_class':FieldArg("cssclass", ""),
                        'odd_class':FieldArg("cssclass", ""),
                        'title1':FieldArg('text', ''),
                        'title2':FieldArg('text', ''),
                        'col1':FieldArgList('text', valdt=False, jsonset=True),
                        'col2':FieldArgList('text', valdt=False, jsonset=True)
                        }

    def __init__(self, name=None, brief='', **field_args):
        """
        header_class: class of the header row, if empty string, then no class will be applied
        even_class: class of even rows, if empty string, then no class will be applied
        odd_class: class of odd rows, if empty string, then no class will be applied
        title1: The header title over the first text column
        title2: The header title over the second text column
        col1 : A list of text strings to place in the first column
        col2: A list of text strings to place in the second column
        """
        Widget.__init__(self, name=name, tag_name="table", brief=brief, **field_args)

    def _build(self, page, ident_list, environ, call_data, lang):
        "Build the table"
        col_list1 = self.get_field_value("col1")
        col_list2 = self.get_field_value("col2")
        header = 0
        if self.get_field_value('title1') or self.get_field_value('title2'):
            header = 1
            if self.get_field_value('header_class'):
                self[0] = tag.Part(tag_name='tr', attribs={"class":self.get_field_value('header_class')})
            else:
                self[0] = tag.Part(tag_name='tr')
            self[0][0] = tag.Part(tag_name='th', text = self.get_field_value('title1'))
            self[0][1] = tag.Part(tag_name='th', text = self.get_field_value('title2'))
        # set even row colour
        if self.get_field_value('even_class'):
            even = self.get_field_value('even_class')
        else:
            even = ''
        # set odd row colour
        if self.get_field_value('odd_class'):
            odd = self.get_field_value('odd_class')
        else:
            odd = ''
        # create rows
        if len(col_list1) == len(col_list2):
            rows = len(col_list1)
        elif len(col_list1) > len(col_list2):
            rows = len(col_list1)
            col_list2.extend(['']*(rows - len(col_list2)))
        else:
            rows = len(col_list2)
            col_list1.extend(['']*(rows - len(col_list1)))
        for index in range(rows):
            rownumber = index+header
            if even and (rownumber % 2) :
                self[rownumber] = tag.Part(tag_name="tr", attribs={"class":even})
            elif odd and not (rownumber % 2):
                self[rownumber] = tag.Part(tag_name='tr', attribs={"class":odd})
            else:
                self[rownumber] = tag.Part(tag_name='tr')
            self[rownumber][0] = tag.Part(tag_name='td', text = col_list1[index])
            self[rownumber][1] = tag.Part(tag_name='td', text = col_list2[index])


    @classmethod
    def description(cls):
        """Returns a text string to illustrate the widget"""
        return """
<table>  <!-- with widget id and class widget_class -->
  <tr> <!-- with header class -->
    <th> <!-- title1 --> </th>
    <th> <!-- title2 --> </th>
  </tr>
  <tr> <!-- with class  from even or odd classes -->
    <td> <!-- col1 text string --> </td>
    <td> <!-- col2 text string --> </td>
  </tr>
  <!-- rows repeated -->
</table>"""


class ThreeColTable1(Widget):
    """A table of three columns, the columns being text strings
       The first row is three header titles
       Note : there is no error display"""

    # This class does not display any error messages
    display_errors = False

    arg_descriptions = {'header_class':FieldArg("cssclass",""),
                        'even_class':FieldArg("cssclass", ""),
                        'odd_class':FieldArg("cssclass", ""),
                        'title1':FieldArg('text', ''),
                        'title2':FieldArg('text', ''),
                        'title3':FieldArg('text', ''),
                        'col1':FieldArgList('text', valdt=False, jsonset=True),
                        'col2':FieldArgList('text', valdt=False, jsonset=True),
                        'col3':FieldArgList('text', valdt=False, jsonset=True)
                        }

    def __init__(self, name=None, brief='', **field_args):
        """
        header_class: class of the header row, if empty string, then no class will be applied
        even_class: class of even rows, if empty string, then no class will be applied
        odd_class: class of odd rows, if empty string, then no class will be applied
        title1: The header title over the first text column
        title2: The header title over the second text column
        title3: The header title over the third text column
        col1 : A list of text strings to place in the first column
        col2: A list of text strings to place in the second column
        col3: A list of text strings to place in the third column
        """
        Widget.__init__(self, name=name, tag_name="table", brief=brief, **field_args)

    def _build(self, page, ident_list, environ, call_data, lang):
        "Build the table"
        col_list1 = self.get_field_value("col1")
        col_list2 = self.get_field_value("col2")
        col_list3 = self.get_field_value("col3")
        header = 0
        if self.get_field_value('title1') or self.get_field_value('title2') or self.get_field_value('title3'):
            header = 1
            if self.get_field_value('header_class'):
                self[0] = tag.Part(tag_name='tr', attribs={"class":self.get_field_value('header_class')})
            else:
                self[0] = tag.Part(tag_name='tr')
            self[0][0] = tag.Part(tag_name='th', text = self.get_field_value('title1'))
            self[0][1] = tag.Part(tag_name='th', text = self.get_field_value('title2'))
            self[0][2] = tag.Part(tag_name='th', text = self.get_field_value('title3'))
        # set even row colour
        if self.get_field_value('even_class'):
            even = self.get_field_value('even_class')
        else:
            even = ''
        # set odd row colour
        if self.get_field_value('odd_class'):
            odd = self.get_field_value('odd_class')
        else:
            odd = ''
        # create rows
        rows = max( len(col_list1), len(col_list2), len(col_list3) )
        if rows > len(col_list1):
            col_list1.extend(['']*(rows - len(col_list1)))
        if rows > len(col_list2):
            col_list2.extend(['']*(rows - len(col_list2)))
        if rows > len(col_list3):
            col_list3.extend(['']*(rows - len(col_list3)))

        for index in range(rows):
            rownumber = index+header
            if even and (rownumber % 2) :
                self[rownumber] = tag.Part(tag_name="tr", attribs={"class":even})
            elif odd and not (rownumber % 2):
                self[rownumber] = tag.Part(tag_name='tr', attribs={"class":odd})
            else:
                self[rownumber] = tag.Part(tag_name='tr')
            self[rownumber][0] = tag.Part(tag_name='td', text = col_list1[index])
            self[rownumber][1] = tag.Part(tag_name='td', text = col_list2[index])
            self[rownumber][2] = tag.Part(tag_name='td', text = col_list3[index])

    @classmethod
    def description(cls):
        """Returns a text string to illustrate the widget"""
        return """
<table>  <!-- with widget id and class widget_class -->
  <tr> <!-- with header class -->
    <th> <!-- title1 --> </th>
    <th> <!-- title2 --> </th>
    <th> <!-- title3 --> </th>
  </tr>
  <tr> <!-- with class  from even or odd classes -->
    <td> <!-- col1 text string --> </td>
    <td> <!-- col2 text string --> </td>
    <td> <!-- col3 text string --> </td>
  </tr>
  <!-- rows repeated -->
</table>"""


class TextBlockTable2(Widget):
    """A table of two columns, the first column being text strings
       the second column being TextBlocks
       The first row is two header titles
       Note : on error the whole widget is shown as errored, there is no error paragraph"""

    # This class does not display any error messages
    display_errors = False

    arg_descriptions = {'header_class':FieldArg("cssclass",""),
                        'even_class':FieldArg("cssclass", ""),
                        'odd_class':FieldArg("cssclass", ""),
                        'title1':FieldArg('text', ''),
                        'title2':FieldArg('text', ''),
                        'contents':FieldArgTable(['text', 'textblock_ref'])
                        }

    def __init__(self, name=None, brief='', **field_args):
        """
        header_class: class of the header row, if empty string, then no class will be applied
        even_class: class of even rows, if empty string, then no class will be applied
        odd_class: class of odd rows, if empty string, then no class will be applied
        title1: The header title over the first text column
        title2: The header title over the second text column
        contents: col 0 is the text to place in the first column,
                  col 1 is the textblock to place in the second colum
        """
        Widget.__init__(self, name=name, tag_name="table", brief=brief, **field_args)


    def _build(self, page, ident_list, environ, call_data, lang):
        "Build the table"
        fieldtable = self.get_field_value("contents")
        header = 0
        if self.get_field_value('title1') or self.get_field_value('title2'):
            header = 1
            if self.get_field_value('header_class'):
                self[0] = tag.Part(tag_name='tr', attribs={"class":self.get_field_value('header_class')})
            else:
                self[0] = tag.Part(tag_name='tr')
            self[0][0] = tag.Part(tag_name='th', text = self.get_field_value('title1'))
            self[0][1] = tag.Part(tag_name='th', text = self.get_field_value('title2'))
        # set even row colour
        if self.get_field_value('even_class'):
            even = self.get_field_value('even_class')
        else:
            even = ''
        # set odd row colour
        if self.get_field_value('odd_class'):
            odd = self.get_field_value('odd_class')
        else:
            odd = ''
        # create rows
        for index, row in enumerate(fieldtable):
            rownumber = index+header
            if even and (rownumber % 2) :
                self[rownumber] = tag.Part(tag_name="tr", attribs={"class":even})
            elif odd and not (rownumber % 2):
                self[rownumber] = tag.Part(tag_name='tr', attribs={"class":odd})
            else:
                self[rownumber] = tag.Part(tag_name='tr')
            self[rownumber][0] = tag.Part(tag_name='td', text = row[0])
            self[rownumber][1] = tag.Part(tag_name='td', text = row[1])


    @classmethod
    def description(cls):
        """Returns a text string to illustrate the widget"""
        return """
<table>  <!-- with widget id and class widget_class -->
  <tr> <!-- with header class -->
    <th> <!-- title1 --> </th>
    <th> <!-- title2 --> </th>
  </tr>
  <tr> <!-- with class  from even or odd classes -->
    <td> <!-- text string --> </td>
    <td> <!-- textblock --> </td>
  </tr>
  <!-- rows repeated -->
</table>"""



class ButtonTextBlockTable1(Widget):
    "A form consisting of a table of form buttons and textblocks"

    error_location = (0,0,0)

    arg_descriptions = {'action':FieldArg("url", ""),
                        'header_class':FieldArg("cssclass",""),
                        'even_class':FieldArg("cssclass", ""),
                        'odd_class':FieldArg("cssclass", ""),
                        'col1_button_title':FieldArg('text', ''),
                        'col2_text_title':FieldArg('text', ''),
                        'hidden_field1':FieldArg("text", valdt=True),
                        'hidden_field2':FieldArg("text", valdt=True),
                        'hidden_field3':FieldArg("text", valdt=True),
                        'hidden_field4':FieldArg("text", valdt=True),
                        'buttons':FieldArgTable(['text', 'textblock_ref'], valdt=True),
                        'button_class':FieldArg("cssclass", ''),
                        'error_class':FieldArg("cssclass", ''),
                        'table_class':FieldArg("cssclass", '')
                        }

    def __init__(self, name=None, brief='', **field_args):
        """
        action: The target page link ident, url or label
        header_class: class of the header row, if empty string, then no class will be applied
        even_class: class of even rows, if empty string, then no class will be applied
        odd_class: class of odd rows, if empty string, then no class will be applied
        button_class: class set on the buttons
        col1_button_title: The title of the first column, above the buttons
        col2_text_title: The title of the second colum, above the textblock
        hidden_field1-4: four hidden fields
        buttons: field name is the button name,
                 text is the values on the button, and which will be returned
                 textbock_ref is the reference of the textblock appearing in the second column
        """
        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
        # hidden error
        self[0] = tag.Part(tag_name="div", attribs={"style":"display:none;"})
        self[0][0] = tag.Part(tag_name="p")
        self[0][0][0] = ''
        # The form
        self[1] = tag.Part(tag_name='form', attribs={"role":"form", "method":"post"})

        # The location 1,0 is the table
        self[1][0] = tag.Part(tag_name="table")


    def _build(self, page, ident_list, environ, call_data, lang):
        if self.get_field_value('error_class'):
            self[0].update_attribs({"class":self.get_field_value('error_class')})
        if self.error_status:
            self[0].del_one_attrib("style")
        if not self.get_field_value("action"):
            # setting self._error replaces the entire tag
            self._error = "Warning: No form action"
            return
        actionurl = skiboot.get_url(self.get_field_value("action"),  proj_ident=page.proj_ident)
        if not actionurl:
            # setting self._error replaces the entire tag
            self._error = "Warning: broken link"
            return
        # update the action of the form
        self[1].update_attribs({"action": actionurl})

        # build the table
        table = self[1][0]
        if self.get_field_value('table_class'):
            table.update_attribs({"class":self.get_field_value('table_class')})

        title1 = self.get_field_value("col1_button_title")
        title2 = self.get_field_value("col2_text_title")

        # set even row colour
        if self.get_field_value('even_class'):
            evenclass = self.get_field_value('even_class')
        else:
            evenclass = ''
        # set odd row colour
        if self.get_field_value('odd_class'):
            oddclass = self.get_field_value('odd_class')
        else:
            oddclass = ''
        # set header row colour
        if self.get_field_value('header_class'):
            headerclass = self.get_field_value('header_class')
        else:
            headerclass = ''
        # set button class
        if self.get_field_value('button_class'):
            buttonclass = self.get_field_value('button_class')
        else:
            buttonclass = ''

        rows = self.get_field_value('buttons')

        for num, row in enumerate(rows):
            if title1 or title2:
                rownumber = num+1
            else:
                rownumber = num
            if (not num) and (title1 or title2):
                if headerclass:
                    table[0]  = tag.Part(tag_name="tr", attribs={"class":headerclass})
                else:
                    table[0]  = tag.Part(tag_name="tr")
                if title1:
                    table[0][0] = tag.Part(tag_name="th", text=title1)
                else:
                    table[0][0] = tag.Part(tag_name="th")
                if title2:
                    table[0][1] = tag.Part(tag_name="th", text=title2)
                else:
                    table[0][1] = tag.Part(tag_name="th")
            if evenclass and (rownumber % 2):
                table[rownumber]  = tag.Part(tag_name="tr", attribs={"class":evenclass})
            elif oddclass and not (rownumber % 2):
                table[rownumber]  = tag.Part(tag_name="tr", attribs={"class":oddclass})
            else:
                table[rownumber]  = tag.Part(tag_name="tr")
            # each row has parameters 0 and 1, and two columns
            table[rownumber][0] = tag.Part(tag_name="td")
            table[rownumber][1] = tag.Part(tag_name="td")
            # submit button
            if buttonclass:
                table[rownumber][0][0]= tag.ClosedPart(tag_name="input",
                                                    attribs = {"value":row[0],
                                                    "type":"submit",
                                                    "name":self.get_formname('buttons'),
                                                    "class":buttonclass})
            else:
                table[rownumber][0][0]= tag.ClosedPart(tag_name="input",
                                                    attribs = {"value":row[0],
                                                    "type":"submit",
                                                    "name":self.get_formname('buttons')})
            # textblock
            table[rownumber][1][0] = row[1]

        # add ident and four hidden fields
        self.add_hiddens(self[1], page)


    @classmethod
    def description(cls):
        """Returns a text string to illustrate the widget"""
        return """
<div> <!-- with widget id and class widget_class -->
  <div> <!-- normally hidden paragraph, with class error_class -->
    <p> <!-- Any error text appears here --> </p>
  </div>
  <form method="post">  <!-- with CSS class given by widget_class, action given by action field -->
    <table>  <!-- with CSS class given by table_class -->
      <tr> <!-- with header class -->
        <th> <!-- title1 --> </th>
        <th> <!-- title2 --> </th>
      </tr>
      <!-- Then the following row repeated for each row given in the buttons field -->
      <tr> <!-- with class  from even or odd classes -->
        <td>
          <input type=\"submit\" />
               <!-- button name set to name of buttons field which forms the widgfield returned -->
              <!-- button value set to buttons row text value and is the value returned when the button is pressed -->
        </td>
        <td> <!-- textblock, found from row textbock_ref --> </td>
      </tr>
      <!-- rows repeated -->
    </table>
    <!-- hidden fields -->
  </form>
</div>"""





# This skis project is added to other projects and consists of
# javascript and css files. As it is essentially a 'library'
# it is usually given a URL of /lib when added - but this is
# not a necessity, just convention.


from skipole import WSGIApplication

PROJECT = 'skis'

def start_call(called_ident, skicall):
    "This project has no start_call functionality"
    return called_ident

def submit_data(skicall):
    "This project has no submit_data functionality"
    return

def end_call(page_ident, page_type, skicall):
    "This project has no end_call functionality"
    return

# As this project is not intended to run as a stand-alone service, a function
# is provided rather than an application object being immediately created.

def makeapp(projectfiles, **proj_data):
    """This function returns the skis application."""

    # The WSGIApplication created here is generally given a URL of "/lib"
    # when added to the root project using application.add_project

    application = WSGIApplication(PROJECT, projectfiles, proj_data, start_call, submit_data, end_call)
    return application








"""Contains widgets for inputting text"""

from string import Template
import json

from .. import skiboot, tag, excepts
from . import Widget, ClosedWidget, FieldArg, FieldArgList, FieldArgTable, FieldArgDict


class TextInput1(ClosedWidget):
    """Defines an input field only (no form, label or submit button)
       does not display errors"""

    # This class does not display any error messages
    display_errors = False

    # js_validators is a class attribute, True if javascript validation is enabled
    js_validators=True

    arg_descriptions = {
                        'input_accepted_class':FieldArg("cssclass", ''),
                        'input_errored_class':FieldArg("cssclass", ''),
                        'set_input_accepted':FieldArg("boolean", False, jsonset=True),
                        'set_input_errored':FieldArg("boolean", False, jsonset=True),
                        'input_text':FieldArg("text", '', valdt=True, jsonset=True),
                        'size':FieldArg("text", ''),
                        'maxlength':FieldArg("text", ''),
                        'disabled':FieldArg("boolean", False),
                        'required':FieldArg("boolean", False),
                        'type':FieldArg("text", 'text'),
                        'pattern':FieldArg("text", ''),
                        'title':FieldArg("text", '')
                       }
    def __init__(self, name=None, brief='', **field_args):
        """
        input_accepted_class: A class which can be set on the input field
        input_errored_class: A class which can be set on the input field
        set_input_accepted: If True, input_accepted_class will be set on the input field
        set_errored_accepted: If True, input_errored_class will be set on the input field
        input_text: The default text in the text input field, field name used as name attribute
        size: The number of characters appearing in the text input field
        maxlength: The maximum number of characters accepted in the text input field
        disabled: Set True if the field is to be disabled
        required: Set True to put the 'required' attribute into the input field
        type: the type set on the input field, such as text or email
        pattern: regular expression pattern
        title: helps describe the pattern
        """
        ClosedWidget.__init__(self, name=name, tag_name="input", brief=brief, **field_args)
        self.update_attribs({"type":"text"})


    def _build(self, page, ident_list, environ, call_data, lang):
        "build the input field"
        self.update_attribs({"name":self.get_formname('input_text'), "value":self.get_field_value('input_text')})
        if self.get_field_value('set_input_accepted') and self.get_field_value('input_accepted_class'):
            if self.get_field_value('widget_class'):
                new_class = self.get_field_value('widget_class') + " " + self.get_field_value('input_accepted_class')
            else:
                new_class = self.get_field_value('input_accepted_class')
            self.update_attribs({"class":new_class})
        if self.get_field_value('set_input_errored') and self.get_field_value('input_errored_class'):
            if self.get_field_value('widget_class'):
                new_class = self.get_field_value('widget_class') + " " + self.get_field_value('input_errored_class')
            else:
                new_class = self.get_field_value('input_errored_class')
            self.update_attribs({"class":new_class})
        if self.get_field_value('size'):
            self.update_attribs({"size":self.get_field_value('size')})
        if self.get_field_value('maxlength'):
            self.update_attribs({"maxlength":self.get_field_value('maxlength')})
        if self.get_field_value('disabled'):
            self.update_attribs({"disabled":"disabled"})
        if self.get_field_value('required'):
            self.update_attribs({"required":"required"})
        if self.get_field_value('type'):
            self.update_attribs({"type":self.get_field_value('type')})
        if self.get_field_value('pattern'):
            self.update_attribs({"pattern":self.get_field_value('pattern')})
        if self.get_field_value('title'):
            self.update_attribs({"title":self.get_field_value('title')})


    def _build_js(self, page, ident_list, environ, call_data, lang):
        """Sets input_accepted_class, input_errored_class into fieldvalues"""
        return self._make_fieldvalues('input_accepted_class', 'input_errored_class')

 
    @classmethod
    def description(cls):
        """Returns a text string to illustrate the widget"""
        return """
  <input type='text' />
  <!-- with widget id and class widget_class and input_accepted_class added if
       set_input_accepted is set to True, and input_errored_class added if
       set_input_errored is set to True, (which will remove input_accepted_class) -->
"""


class Password1(ClosedWidget):
    """Defines a password input field only (no form, label or submit button)
       does not display errors"""

    # This class does not display any error messages
    display_errors = False

    # js_validators is a class attribute, True if javascript validation is enabled
    js_validators=True

    arg_descriptions = {
                        'input_accepted_class':FieldArg("cssclass", ''),
                        'input_errored_class':FieldArg("cssclass", ''),
                        'set_input_accepted':FieldArg("boolean", False, jsonset=True),
                        'set_input_errored':FieldArg("boolean", False, jsonset=True),
                        'input_text':FieldArg("text", '', valdt=True, jsonset=True),
                        'size':FieldArg("text", ''),
                        'maxlength':FieldArg("text", ''),
                        'disabled':FieldArg("boolean", False),
                        'required':FieldArg("boolean", False)
                       }
    def __init__(self, name=None, brief='', **field_args):
        """
        input_accepted_class: A class which can be set on the input field
        input_errored_class: A class which can be set on the input field
        set_input_accepted: If True, input_accepted_class will be set on the input field
        set_errored_accepted: If True, input_errored_class will be set on the input field
        input_text: The default text in the text input field, field name used as name attribute
        size: The number of characters appearing in the text input field
        maxlength: The maximum number of characters accepted in the text input field
        disabled: Set True if the field is to be disabled
        required: Set True to put the 'required' attribute into the input field
        """
        ClosedWidget.__init__(self, name=name, tag_name="input", brief=brief, **field_args)
        self.update_attribs({"type":"password"})


    def _build(self, page, ident_list, environ, call_data, lang):
        "build the input field"
        self.update_attribs({"name":self.get_formname('input_text'), "value":self.get_field_value('input_text')})
        if self.get_field_value('set_input_accepted') and self.get_field_value('input_accepted_class'):
            if self.get_field_value('widget_class'):
                new_class = self.get_field_value('widget_class') + " " + self.get_field_value('input_accepted_class')
            else:
                new_class = self.get_field_value('input_accepted_class')
            self.update_attribs({"class":new_class})
        if self.get_field_value('set_input_errored') and self.get_field_value('input_errored_class'):
            if self.get_field_value('widget_class'):
                new_class = self.get_field_value('widget_class') + " " + self.get_field_value('input_errored_class')
            else:
                new_class = self.get_field_value('input_errored_class')
            self.update_attribs({"class":new_class})
        if self.get_field_value('size'):
            self.update_attribs({"size":self.get_field_value('size')})
        if self.get_field_value('maxlength'):
            self.update_attribs({"maxlength":self.get_field_value('maxlength')})
        if self.get_field_value('disabled'):
            self.update_attribs({"disabled":"disabled"})
        if self.get_field_value('required'):
            self.update_attribs({"required":"required"})


    def _build_js(self, page, ident_list, environ, call_data, lang):
        """Sets input_accepted_class, input_errored_class into fieldvalues"""
        return self._make_fieldvalues('input_accepted_class', 'input_errored_class')

 
    @classmethod
    def description(cls):
        """Returns a text string to illustrate the widget"""
        return """
  <input type='password' />
  <!-- with widget id and class widget_class and input_accepted_class added if
       set_input_accepted is set to True, and input_errored_class added if
       set_input_errored is set to True, (which will remove input_accepted_class) -->
"""


class TextInput2(Widget):
    """Defines a div containing a hidden error paragraph
       with a text input field (no form or submit button)
       A label to the left of the text can have a class and text set"""

    # js_validators is a class attribute, True if javascript validation is enabled
    js_validators=True

    error_location = (0,0,0)

    arg_descriptions = {
                        'input_accepted_class':FieldArg("cssclass", ''),
                        'input_errored_class':FieldArg("cssclass", ''),
                        'inputdiv_class':FieldArg("cssclass", ''),
                        'set_input_accepted':FieldArg("boolean", False, jsonset=True),
                        'set_input_errored':FieldArg("boolean", False, jsonset=True),
                        'label':FieldArg("text", 'Your Input:'),
                        'label_class':FieldArg("cssclass", ''),
                        'label_style':FieldArg("cssstyle", ''),
                        'error_class':FieldArg("cssclass", ''),
                        'input_text':FieldArg("text", '', valdt=True, jsonset=True),
                        'input_class':FieldArg("cssclass", ''),
                        'input_disabled_class':FieldArg("cssclass", ''),
                        'size':FieldArg("text", ''),
                        'maxlength':FieldArg("text", ''),
                        'redstar':FieldArg("boolean", False),
                        'redstar_class':FieldArg("cssclass", ''),
                        'redstar_style':FieldArg("cssstyle", ''),
                        'disabled':FieldArg("boolean", False, jsonset=True),
                        'required':FieldArg("boolean", False),
                        'type':FieldArg("text", 'text'),
                        'pattern':FieldArg("text", ''),
                        'title':FieldArg("text", '')
                       }
    def __init__(self, name=None, brief='', **field_args):
        """
        input_accepted_class: A class which can be set on the input field
        input_errored_class: A class which can be set on the input field
        set_input_accepted: If True, input_accepted_class will be set on the input field
        set_errored_accepted: If True, input_errored_class will be set on the input field
        label: The text appearing in a label tag to the left of the input field
        label_class: The css class of the label to the left of the input field
        error_class: The class applied to the hidden paragraph on error.
        inputdiv_class: The css class of the div containing the label and input field
        input_text: The default text in the text input field, field name used as name attribute
        input_class: The css class of the input field
        input_disabled_class: css class of the input field which replaces input_class if disabled is True
        size: The number of characters appearing in the text input area
        maxlength: The maximum number of characters accepted in the text area
        redstar: If True a red asterix is shown by the side of the input field
        disabled: Set True if the field is to be disabled
        required: Set True to put the 'required' flag into the input field
        type: the type set on the input field, such as text or email
        pattern: regular expression pattern
        title: helps describe the pattern
        """
        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
        # error div at 0
        self[0] = tag.Part(tag_name="div", attribs={"style":"display:none;"})
        self[0][0] = tag.Part(tag_name="p")
        self[0][0][0] = ''
        # inputdiv
        self[1] = tag.Part(tag_name="div")
        # the label
        self[1][0] = tag.Part(tag_name="label", hide_if_empty=True)
        # the text input field
        self[1][1] = tag.ClosedPart(tag_name="input", attribs ={"type":"text"})


    def _build(self, page, ident_list, environ, call_data, lang):
        "build the input field"
        if self.get_field_value('error_class'):
            self[0].update_attribs({"class":self.get_field_value('error_class')})
        if self.error_status:
            self[0].del_one_attrib("style")
        if self.get_field_value('inputdiv_class'):
            self[1].update_attribs({"class": self.get_field_value('inputdiv_class')})
        if self.get_field_value('label_class'):
            self[1][0].update_attribs({"class": self.get_field_value('label_class')})
        if self.get_field_value('label_style'):
            self[1][0].update_attribs({"style": self.get_field_value('label_style')})
        if self.get_field_value("label"):
            self[1][0][0] = self.get_field_value("label")
        # set an id in the input field for the 'label for' tag
        self[1][1].insert_id()
        self[1][0].update_attribs({'for':self[1][1].get_id()})
        self[1][1].update_attribs({"name":self.get_formname('input_text'), "value":self.get_field_value('input_text')})
        if self.get_field_value('size'):
            self[1][1].update_attribs({"size":self.get_field_value('size')})
        if self.get_field_value('maxlength'):
            self[1][1].update_attribs({"maxlength":self.get_field_value('maxlength')})
        if self.get_field_value('required'):
            self[1][1].update_attribs({"required":"required"})
        if self.get_field_value('type'):
            self[1][1].update_attribs({"type":self.get_field_value('type')})
        if self.get_field_value('pattern'):
            self[1][1].update_attribs({"pattern":self.get_field_value('pattern')})
        if self.get_field_value('title'):
            self[1][1].update_attribs({"title":self.get_field_value('title')})
        if self.get_field_value('input_class'):
            input_class = self.get_field_value('input_class')
        else:
            input_class = ''
        if self.get_field_value('disabled'):
            self[1][1].update_attribs({"disabled":"disabled"})
            if self.get_field_value('input_disabled_class'):
                input_class = self.get_field_value('input_disabled_class')
        if self.error_status and self.get_field_value('input_errored_class'):
            if input_class:
                input_class = input_class + ' ' + self.get_field_value('input_errored_class')
            else:
                input_class = self.get_field_value('input_errored_class')
        elif self.get_field_value('set_input_errored') and self.get_field_value('input_errored_class'):
            if input_class:
                input_class = input_class + ' ' + self.get_field_value('input_errored_class')
            else:
                input_class = self.get_field_value('input_errored_class')
        elif self.get_field_value('set_input_accepted') and self.get_field_value('input_accepted_class'):
            if input_class:
                input_class = input_class + ' ' + self.get_field_value('input_accepted_class')
            else:
                input_class = self.get_field_value('input_accepted_class')
        if input_class:
            self[1][1].update_attribs({"class":input_class})
        # redstar
        if self.get_field_value('redstar'):
            self[1][2] = tag.Part(tag_name="span")
            if self.get_field_value('redstar_style'):
                self[1][2].update_attribs({"style":self.get_field_value('redstar_style')})
            if self.get_field_value('redstar_class'):
                self[1][2].update_attribs({"class":self.get_field_value('redstar_class')})
            self[1][2][0] = '*'


    def _build_js(self, page, ident_list, environ, call_data, lang):
        """Sets input classes into fieldvalues"""
        fieldlist = []
        if self.get_field_value('input_accepted_class'):
            fieldlist.append('input_accepted_class')
        if self.get_field_value('input_errored_class'):
            fieldlist.append('input_errored_class')
        if self.get_field_value('input_class'):
            fieldlist.append('input_class')
        if self.get_field_value('input_disabled_class'):
            fieldlist.append('input_disabled_class')
        if fieldlist:
            return self._make_fieldvalues(*fieldlist)
        return ''

    @classmethod
    def description(cls):
        """Returns a text string to illustrate the widget"""
        return """
<div>  <!-- with widget id and class widget_class -->
  <div> <!-- normally hidden div, with class error_class -->
    <p> <!-- Any error text appears here --> </p>
  </div>
  <div>  <!-- class attribute set to inputdiv_class -->
    <label> <!-- with class set to label_class -->
            <!-- content set to label -->
    </label>
    <input type='text' /> <!-- with attributes set appropriately -->
    <span>*</span> <!-- shown if redstar is True -->
  </div>
</div>"""


class Password2(Widget):
    """Defines a div containing a hidden error paragraph
       with a password input field (no form or submit button)
       A label to the left of the text can have a class and text set"""

    # js_validators is a class attribute, True if javascript validation is enabled
    js_validators=True

    error_location = (0,0,0)

    arg_descriptions = {
                        'input_accepted_class':FieldArg("cssclass", ''),
                        'input_errored_class':FieldArg("cssclass", ''),
                        'input_disabled_class':FieldArg("cssclass", ''),
                        'inputdiv_class':FieldArg("cssclass", ''),
                        'set_input_accepted':FieldArg("boolean", False, jsonset=True),
                        'set_input_errored':FieldArg("boolean", False, jsonset=True),
                        'label':FieldArg("text", 'Password:'),
                        'label_class':FieldArg("cssclass", ''),
                        'label_style':FieldArg("cssstyle", ''),
                        'error_class':FieldArg("cssclass", ''),
                        'input_text':FieldArg("text", '', valdt=True, jsonset=True),
                        'input_class':FieldArg("cssclass", ''),
                        'input_style':FieldArg("cssstyle", ''),
                        'size':FieldArg("text", ''),
                        'maxlength':FieldArg("text", ''),
                        'disabled':FieldArg("boolean", False, jsonset=True),
                        'redstar':FieldArg("boolean", False),
                        'redstar_class':FieldArg("cssclass", ''),
                        'redstar_style':FieldArg("cssstyle", ''),
                        'required':FieldArg("boolean", False)
                       }
    def __init__(self, name=None, brief='', **field_args):
        """
        input_accepted_class: A class which can be set on the input field
        input_errored_class: A class which can be set on the input field
        input_disabled_class: css class of the input field which replaces input_class if disabled is True
        inputdiv_class: The css class of the div containing the label and input field
        set_input_accepted: If True, input_accepted_class will be set on the input field
        set_errored_accepted: If True, input_errored_class will be set on the input field
        label: The text appearing in a label tag to the left of the input field
        label_class: The css class of the label to the left of the input field
        error_class: The class applied to the hidden paragraph on error.
        input_text: The default text in the text input field, field name used as name attribute
        input_class: The css class of the input field
        size: The number of characters appearing in the text input area
        maxlength: The maximum number of characters accepted in the text area
        disabled: Set True if the field is to be disabled
        redstar: If True a red asterix is shown by the side of the input field
        required: Set True to put the 'required' flag into the input field
        """
        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
        # error div at 0
        self[0] = tag.Part(tag_name="div", attribs={"style":"display:none;"})
        self[0][0] = tag.Part(tag_name="p")
        self[0][0][0] = ''
        # inputdiv
        self[1] = tag.Part(tag_name="div")
        # the label
        self[1][0] = tag.Part(tag_name="label", hide_if_empty=True)
        # the text input field
        self[1][1] = tag.ClosedPart(tag_name="input", attribs ={"type":"password"})


    def _build(self, page, ident_list, environ, call_data, lang):
        "build the input field"
        if self.get_field_value('error_class'):
            self[0].update_attribs({"class":self.get_field_value('error_class')})
        if self.error_status:
            self[0].del_one_attrib("style")
        if self.get_field_value('inputdiv_class'):
            self[1].update_attribs({"class": self.get_field_value('inputdiv_class')})
        if self.get_field_value('label_class'):
            self[1][0].update_attribs({"class": self.get_field_value('label_class')})
        if self.get_field_value('label_style'):
            self[1][0].update_attribs({"style": self.get_field_value('label_style')})
        if self.get_field_value("label"):
            self[1][0][0] = self.get_field_value("label")
        # set an id in the input field for the 'label for' tag
        self[1][1].insert_id()
        self[1][0].update_attribs({'for':self[1][1].get_id()})
        self[1][1].update_attribs({"name":self.get_formname('input_text'), "value":self.get_field_value('input_text')})
        if self.get_field_value('size'):
            self[1][1].update_attribs({"size":self.get_field_value('size')})
        if self.get_field_value('maxlength'):
            self[1][1].update_attribs({"maxlength":self.get_field_value('maxlength')})
        if self.get_field_value('required'):
            self[1][1].update_attribs({"required":"required"})
        if self.get_field_value('input_style'):
            self[1][1].update_attribs({"style":self.get_field_value('input_style')})
        if self.get_field_value('input_class'):
            input_class = self.get_field_value('input_class')
        else:
            input_class = ''
        if self.get_field_value('disabled'):
            self[1][1].update_attribs({"disabled":"disabled"})
            if self.get_field_value('input_disabled_class'):
                input_class = self.get_field_value('input_disabled_class')
        if self.error_status and self.get_field_value('input_errored_class'):
            if input_class:
                input_class = input_class + ' ' + self.get_field_value('input_errored_class')
            else:
                input_class = self.get_field_value('input_errored_class')
        elif self.get_field_value('set_input_errored') and self.get_field_value('input_errored_class'):
            if input_class:
                input_class = input_class + ' ' + self.get_field_value('input_errored_class')
            else:
                input_class = self.get_field_value('input_errored_class')
        elif self.get_field_value('set_input_accepted') and self.get_field_value('input_accepted_class'):
            if input_class:
                input_class = input_class + ' ' + self.get_field_value('input_accepted_class')
            else:
                input_class = self.get_field_value('input_accepted_class')
        if input_class:
            self[1][1].update_attribs({"class":input_class})
        # redstar
        if self.get_field_value('redstar'):
            self[1][2] = tag.Part(tag_name="span")
            if self.get_field_value('redstar_style'):
                self[1][2].update_attribs({"style":self.get_field_value('redstar_style')})
            if self.get_field_value('redstar_class'):
                self[1][2].update_attribs({"class":self.get_field_value('redstar_class')})
            self[1][2][0] = '*'



    def _build_js(self, page, ident_list, environ, call_data, lang):
        """Sets input classes into fieldvalues"""
        fieldlist = []
        if self.get_field_value('input_accepted_class'):
            fieldlist.append('input_accepted_class')
        if self.get_field_value('input_errored_class'):
            fieldlist.append('input_errored_class')
        if self.get_field_value('input_class'):
            fieldlist.append('input_class')
        if self.get_field_value('input_disabled_class'):
            fieldlist.append('input_disabled_class')
        if fieldlist:
            return self._make_fieldvalues(*fieldlist)
        return ''


    @classmethod
    def description(cls):
        """Returns a text string to illustrate the widget"""
        return """
<div>  <!-- with widget id and class widget_class -->
  <div> <!-- normally hidden paragraph, with class error_class -->
    <p> <!-- Any error text appears here --> </p>
  </div>
  <div>  <!-- class attribute set to inputdiv_class -->
    <label> <!-- with class set to label_class -->
            <!-- content set to label -->
    </label>
    <input type='password' /> <!-- with attributes set appropriately -->
    <span>*</span> <!-- shown if redstar is True -->
  </div>
</div>"""


class TextInput3(Widget):
    """Defines a div containing a text input field (no form or submit button)
       with a left and right label, does not display errors"""

    # This class does not display any error messages
    display_errors = False

    # js_validators is a class attribute, True if javascript validation is enabled
    js_validators=True

    arg_descriptions = {
                        'input_class':FieldArg("cssclass", ''),
                        'input_style':FieldArg("cssstyle", ''),
                        'input_accepted_class':FieldArg("cssclass", ''),
                        'input_errored_class':FieldArg("cssclass", ''),
                        'set_input_accepted':FieldArg("boolean", False, jsonset=True),
                        'set_input_errored':FieldArg("boolean", False, jsonset=True),
                        'left_label':FieldArg("text", 'Your Input:'),
                        'left_class':FieldArg("cssclass", ''),
                        'left_style':FieldArg("cssstyle", ''),
                        'right_label':FieldArg("text", ''),
                        'right_class':FieldArg("cssclass", ''),
                        'right_style':FieldArg("cssstyle", ''),
                        'input_text':FieldArg("text", '', valdt=True, jsonset=True),
                        'size':FieldArg("text", ''),
                        'maxlength':FieldArg("text", ''),
                        'disabled':FieldArg("boolean", False),
                        'required':FieldArg("boolean", False),
                        'type':FieldArg("text", 'text'),
                        'pattern':FieldArg("text", ''),
                        'title':FieldArg("text", '')
                       }
    def __init__(self, name=None, brief='', **field_args):
        """
        input_class: The css class of the input field
        input_style: The css style of the input field
        input_accepted_class: A class which can be set on the input field
        input_errored_class: A class which can be set on the input field
        set_input_accepted: If True, input_accepted_class will be set on the input field
        set_errored_accepted: If True, input_errored_class will be set on the input field
        left_label: The text displayed to the left of the input field
        left_class: The css class of the label to the left of the input field
        right_label: The text displayed to the right of the input field
        right_class: The css class of the label to the right of the input field
        input_text: The default text in the text input field, field name used as name attribute
        size: The number of characters appearing in the text input field
        maxlength: The maximum number of characters accepted in the text input field
        disabled: Set True if the field is to be disabled
        required: Set True to put the 'required' attribute into the input field
        type: the type set on the input field, such as text or email
        pattern: regular expression pattern
        title: helps describe the pattern
        """
        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
        self[0] = tag.Part(tag_name="label", hide_if_empty=True)
        self[1] = tag.ClosedPart(tag_name="input", attribs = {"type":"text"})
        self[2] = tag.Part(tag_name="label", hide_if_empty=True)


    def _build(self, page, ident_list, environ, call_data, lang):
        "build the input field"
        if self.get_field_value('left_label'):
            self[0][0] = self.get_field_value('left_label')
        if self.get_field_value('left_class'):
            self[0].attribs = {"class": self.get_field_value('left_class')}
        if self.get_field_value('left_style'):
            self[0].update_attribs({"style": self.get_field_value('left_style')})
        self[1].update_attribs({"name":self.get_formname('input_text'), "value":self.get_field_value('input_text')})

        if self.get_field_value('input_class'):
            input_class = self.get_field_value('input_class')
        else:
            input_class = ''

        if self.get_field_value('set_input_errored') and self.get_field_value('input_errored_class'):
            if input_class:
                input_class = input_class + ' ' + self.get_field_value('input_errored_class')
            else:
                input_class = self.get_field_value('input_errored_class')
        elif self.get_field_value('set_input_accepted') and self.get_field_value('input_accepted_class'):
            if input_class:
                input_class = input_class + ' ' + self.get_field_value('input_accepted_class')
            else:
                input_class = self.get_field_value('input_accepted_class')
        if input_class:
            self[1].update_attribs({"class":input_class})

        if self.get_field_value('input_style'):
            self[1].update_attribs({"style":self.get_field_value('input_style')})
        if self.get_field_value('size'):
            self[1].update_attribs({"size":self.get_field_value('size')})
        if self.get_field_value('maxlength'):
            self[1].update_attribs({"maxlength":self.get_field_value('maxlength')})
        if self.get_field_value('disabled'):
            self[1].update_attribs({"disabled":"disabled"})
        if self.get_field_value('required'):
            self[1].update_attribs({"required":"required"})
        if self.get_field_value('type'):
            self[1].update_attribs({"type":self.get_field_value('type')})
        if self.get_field_value('pattern'):
            self[1].update_attribs({"pattern":self.get_field_value('pattern')})
        if self.get_field_value('title'):
            self[1].update_attribs({"title":self.get_field_value('title')})
        if self.get_field_value('right_label'):
            self[2][0] = self.get_field_value('right_label')
        if self.get_field_value('right_class'):
            self[2].attribs = {"class": self.get_field_value('right_class')}
        if self.get_field_value('right_style'):
            self[2].update_attribs({"style": self.get_field_value('right_style')})

        # set an id in the text input field for the 'label for' tag
        self[1].insert_id()
        # set the label 'for' attribute
        self[0].update_attribs({'for':self[1].get_id()})
        self[2].update_attribs({'for':self[1].get_id()})


    def _build_js(self, page, ident_list, environ, call_data, lang):
        """Sets input_accepted_class, input_errored_class into fieldvalues"""
        return self._make_fieldvalues('input_accepted_class', 'input_errored_class')
 
    @classmethod
    def description(cls):
        """Returns a text string to illustrate the widget"""
        return """
<div>  <!-- with widget id and class widget_class -->
  <label> <!-- with class set to left_class and content to left_label -->
  </label>
  <input type='text' /> <!-- with attributes set appropriately -->
  <label> <!-- with class set to right_class and content to right_label -->
  </label>
</div>"""


class TextInput4(Widget):
    """Defines a div containing a div with label
       and a div with a text input field (no form or submit button)
       All divs, label and input field can have class set
       No error display"""

    # This class does not display any error messages
    display_errors = False

    # js_validators is a class attribute, True if javascript validation is enabled
    js_validators=True   

    arg_descriptions = {
                        'input_accepted_class':FieldArg("cssclass", ''),
                        'input_errored_class':FieldArg("cssclass", ''),
                        'set_input_accepted':FieldArg("boolean", False, jsonset=True),
                        'set_input_errored':FieldArg("boolean", False, jsonset=True),
                        'labeldiv_class':FieldArg("cssclass", ''),
                        'labeldiv_style':FieldArg("cssstyle", ''),
                        'label':FieldArg("text", 'Your input:'),
                        'label_class':FieldArg("cssclass", ''),
                        'label_style':FieldArg("cssstyle", ''),
                        'inputdiv_class':FieldArg("cssclass", ''),
                        'inputdiv_style':FieldArg("cssstyle", ''),
                        'input_text':FieldArg("text", '', valdt=True, jsonset=True),
                        'input_class':FieldArg("cssclass", ''),
                        'input_style':FieldArg("cssstyle", ''),
                        'input_disabled_class':FieldArg("cssclass", ''),
                        'size':FieldArg("text", ''),
                        'maxlength':FieldArg("text", ''),
                        'redstar':FieldArg("boolean", False),
                        'redstar_class':FieldArg("cssclass", ''),
                        'redstar_style':FieldArg("cssstyle", ''),
                        'disabled':FieldArg("boolean", False, jsonset=True),
                        'required':FieldArg("boolean", False),
                        'type':FieldArg("text", 'text'),
                        'pattern':FieldArg("text", ''),
                        'title':FieldArg("text", '')
                       }
    def __init__(self, name=None, brief='', **field_args):
        """
        input_accepted_class: A class which can be set on the input field
        input_errored_class: A class which can be set on the input field
        set_input_accepted: If True, input_accepted_class will be set on the input field
        set_errored_accepted: If True, input_errored_class will be set on the input field
        labeldiv_class: The css class of the div containing the label
        label: The text appearing in a label tag to the left of the input field
        label_class: The css class of the label to the left of the input field
        inputdiv_class: The css class of the div containing the input field
        input_text: The default text in the text input field, field name used as name attribute
        input_class: The css class of the input field
        input_disabled_class: css class of the input field which replaces input_class if disabled is True
        size: The number of characters appearing in the text input area
        maxlength: The maximum number of characters accepted in the text area
        redstar: If True a red asterix is shown by the side of the input field
        disabled: Set True if the field is to be disabled
        required: Set True to put the 'required' flag into the input field
        type: the type set on the input field, such as text or email
        pattern: regular expression pattern
        title: helps describe the pattern
        """
        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
        # label div at 0
        self[0] = tag.Part(tag_name="div")
        # the label
        self[0][0] = tag.Part(tag_name="label", hide_if_empty=True)
        # input div at 1
        self[1] = tag.Part(tag_name="div")
        # the text input field
        self[1][0] = tag.ClosedPart(tag_name="input", attribs ={"type":"text"})


    def _build(self, page, ident_list, environ, call_data, lang):
        "build the input field"
        # label
        if self.get_field_value('labeldiv_class'):
            self[0].update_attribs({"class": self.get_field_value('labeldiv_class')})
        if self.get_field_value('labeldiv_style'):
            self[0].update_attribs({"style": self.get_field_value('labeldiv_style')})
        if self.get_field_value('label_class'):
            self[0][0].update_attribs({"class": self.get_field_value('label_class')})
        if self.get_field_value('label_style'):
            self[0][0].update_attribs({"style": self.get_field_value('label_style')})
        if self.get_field_value("label"):
            self[0][0][0] = self.get_field_value("label")
        # input div
        if self.get_field_value('inputdiv_class'):
            self[1].update_attribs({"class": self.get_field_value('inputdiv_class')})
        if self.get_field_value('inputdiv_style'):
            self[1].update_attribs({"style": self.get_field_value('inputdiv_style')})
        # set an id in the input field for the 'label for' tag
        self[1][0].insert_id()
        self[0][0].update_attribs({'for':self[1][0].get_id()})
        # input field
        self[1][0].update_attribs({"name":self.get_formname('input_text'), "value":self.get_field_value('input_text')})
        if self.get_field_value('size'):
            self[1][0].update_attribs({"size":self.get_field_value('size')})
        if self.get_field_value('maxlength'):
            self[1][0].update_attribs({"maxlength":self.get_field_value('maxlength')})
        if self.get_field_value('required'):
            self[1][0].update_attribs({"required":"required"})
        if self.get_field_value('type'):
            self[1][0].update_attribs({"type":self.get_field_value('type')})
        if self.get_field_value('pattern'):
            self[1][0].update_attribs({"pattern":self.get_field_value('pattern')})
        if self.get_field_value('title'):
            self[1][0].update_attribs({"title":self.get_field_value('title')})
        if self.get_field_value('input_style'):
            self[1][0].update_attribs({"style":self.get_field_value('input_style')})
        if self.get_field_value('input_class'):
            input_class = self.get_field_value('input_class')
        else:
            input_class = ''
        if self.get_field_value('disabled'):
            self[1][0].update_attribs({"disabled":"disabled"})
            if self.get_field_value('input_disabled_class'):
                input_class = self.get_field_value('input_disabled_class')
        if self.get_field_value('set_input_errored') and self.get_field_value('input_errored_class'):
            if input_class:
                input_class = input_class + ' ' + self.get_field_value('input_errored_class')
            else:
                input_class = self.get_field_value('input_errored_class')
        elif self.get_field_value('set_input_accepted') and self.get_field_value('input_accepted_class'):
            if input_class:
                input_class = input_class + ' ' + self.get_field_value('input_accepted_class')
            else:
                input_class = self.get_field_value('input_accepted_class')
        if input_class:
            self[1][0].update_attribs({"class":input_class})
        # redstar
        if self.get_field_value('redstar'):
            self[1][1] = tag.Part(tag_name="span")
            self[1][1][0] = '*'
            if self.get_field_value('redstar_style'):
                self[1][1].update_attribs({"style":self.get_field_value('redstar_style')})
            if self.get_field_value('redstar_class'):
                self[1][1].update_attribs({"class":self.get_field_value('redstar_class')})

    def _build_js(self, page, ident_list, environ, call_data, lang):
        """Sets input classes into fieldvalues"""
        fieldlist = []
        if self.get_field_value('input_accepted_class'):
            fieldlist.append('input_accepted_class')
        if self.get_field_value('input_errored_class'):
            fieldlist.append('input_errored_class')
        if self.get_field_value('input_class'):
            fieldlist.append('input_class')
        if self.get_field_value('input_disabled_class'):
            fieldlist.append('input_disabled_class')
        if fieldlist:
            return self._make_fieldvalues(*fieldlist)
        return ''

    @classmethod
    def description(cls):
        """Returns a text string to illustrate the widget"""
        return """
<div>  <!-- with widget id and class widget_class -->
  <div> <!-- with class set to labeldiv_class -->
    <label> <!-- with class set to label_class -->
            <!-- content set to label -->
    </label>
  </div>
  <div> <!-- with class set to inputdiv_class -->
    <input type='text' /> <!-- with attributes set appropriately -->
    <span>*</span> <!-- shown if redstar is True -->
  </div>
</div>"""



class SubmitTextInput1(Widget):
    """Defines a form with a text input field, and four hidden fields"""

    # js_validators is a class attribute, True if javascript validation is enabled
    js_validators=True

    error_location = (0,0,0)

    arg_descriptions = {'label':FieldArg("text", 'Your input:'),
                        'label_class':FieldArg("cssclass", ''),
                        'label_style':FieldArg("cssstyle", ''),
                        'action_json':FieldArg("url", ''),
                        'action':FieldArg("url", ''),
                        'hidden_field1':FieldArg("text", '', valdt=True),
                        'hidden_field2':FieldArg("text", '', valdt=True),
                        'hidden_field3':FieldArg("text", '', valdt=True),
                        'hidden_field4':FieldArg("text", '', valdt=True),
                        'target':FieldArg("text",''),
                        'button_text':FieldArg("text",'Submit'),
                        'button_class':FieldArg("cssclass", ''),
                        'inputdiv_class':FieldArg("cssclass", ''),
                        'inputandbutton_class':FieldArg("cssclass", ''),
                        'inputandbutton_style':FieldArg("cssstyle", ''),
                        'error_class':FieldArg("cssclass", ''),
                        'input_text':FieldArg("text", '', valdt=True, jsonset=True),
                        'size':FieldArg("text", ''),
                        'maxlength':FieldArg("text", ''),
                        'required':FieldArg("boolean", False),
                        'type':FieldArg("text", 'text'),
                        'pattern':FieldArg("text", ''),
                        'title':FieldArg("text", ''),
                        'input_accepted_class':FieldArg("cssclass", ''),
                        'input_errored_class':FieldArg("cssclass", ''),
                        'input_class':FieldArg("cssclass", ''),
                        'set_input_accepted':FieldArg("boolean", False, jsonset=True),
                        'set_input_errored':FieldArg("boolean", False, jsonset=True),
                        'hide':FieldArg("boolean", False, jsonset=True)
                       }

    def __init__(self, name=None, brief='', **field_args):
        """
        label: The text displayed to the left of the text input field
        label_class: The css class of the label
        action_json:  if a value set, and client has jscript enabled, this is the page ident, label, url this button links to, expects a json page back
        action: The page ident, label, url this button links to, overridden if action_json is set.
        hidden_field1: A hidden field value, leave blank if unused, name used as the get field name
        hidden_field2: A second hidden field value, leave blank if unused, name used as the get field name
        hidden_field3: A third hidden field value, leave blank if unused, name used as the get field name
        hidden_field4: A fourth hidden field value, leave blank if unused, name used as the get field name
        target: if given, the target attribute will be set
        button_text: The text on the button
        button_class: The class given to the button tag
        inputdiv_class: the class attribute of the div which contains the label, input text and button
        inputandbutton_class: the class attribute of the span which contains the input text and button
        error_class: The class applied to the paragraph containing the error message on error.
        input_text: The default text in the text input field, field name used as the name attribute
        size: The number of characters appearing in the text input area
        maxlength: The maximum number of characters accepted in the text area
        required: Set True to put the 'required' flag into the input field
        type: the type set on the input field, such as text or email
        pattern: regular expression pattern
        title: helps describe the pattern
        input_accepted_class: A class which can be set on the input field
        input_errored_class: A class which can be set on the input field
        input_class: Class set on the input field
        set_input_accepted: If True, input_accepted_class will be set on the input field
        set_errored_accepted: If True, input_errored_class will be set on the input field
        hide: If True, widget is hidden
        """
        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
        # error div at 0
        self[0] = tag.Part(tag_name="div", attribs={"style":"display:none;"})
        self[0][0] = tag.Part(tag_name="p")
        self[0][0][0] = ''
        # The form
        self[1] = tag.Part(tag_name='form', attribs={"role":"form", "method":"post"})

        # div containing label, input text and button
        self[1][0] = tag.Part(tag_name='div')
        # the label
        self[1][0][0] = tag.Part(tag_name="label", hide_if_empty=True)
        # span containing input text and button
        self[1][0][1] = tag.Part(tag_name='span')
        # the text input field
        self[1][0][1][0] = tag.ClosedPart(tag_name="input", attribs ={"type":"text"})
        # the submit button
        self[1][0][1][1] = tag.Part(tag_name="button", attribs ={"type":"submit"})
        self[1][0][1][1][0] = "Submit"
        self._jsonurl = ''


    def _build(self, page, ident_list, environ, call_data, lang):
        "build the form"
        if self.get_field_value("target"):
            self[1].update_attribs({"target":self.get_field_value("target")})
        # Hides widget if no error and hide is True
        self.widget_hide(self.get_field_value("hide"))
        self._jsonurl = skiboot.get_url(self.get_field_value("action_json"), proj_ident=page.proj_ident)
        if self.get_field_value('error_class'):
            self[0].update_attribs({"class":self.get_field_value('error_class')})
        if self.error_status:
            self[0].del_one_attrib("style")
        if not self.get_field_value("action"):
            # setting self._error replaces the entire tag
            self._error = "Warning: No form action"
            return
        actionurl = skiboot.get_url(self.get_field_value("action"),  proj_ident=page.proj_ident)
        if not actionurl:
            # setting self._error replaces the entire tag
            self._error = "Warning: broken link"
            return
        # update the action of the form
        self[1].update_attribs({"action": actionurl})
        # the div holding label, input text and button
        if self.get_field_value('inputdiv_class'):
            self[1][0].attribs = {"class": self.get_field_value('inputdiv_class')}

        if self.get_field_value('label_class'):
            self[1][0][0].attribs = {"class": self.get_field_value('label_class')}
        if self.get_field_value('label_style'):
            self[1][0][0].attribs = {"style": self.get_field_value('label_style')}
        if self.get_field_value('label'):
            self[1][0][0][0] = self.get_field_value('label')

        # the span holding input text and button
        if self.get_field_value('inputandbutton_class'):
            self[1][0][1].attribs = {"class": self.get_field_value('inputandbutton_class')}
        if self.get_field_value('inputandbutton_style'):
            self[1][0][1].attribs = {"style": self.get_field_value('inputandbutton_style')}

        # set an id in the input field for the 'label for' tag
        self[1][0][1][0].insert_id()

        self[1][0][1][0].update_attribs({"name":self.get_formname('input_text'), "value":self.get_field_value('input_text')})
        if self.get_field_value('size'):
            self[1][0][1][0].update_attribs({"size":self.get_field_value('size')})
        if self.get_field_value('maxlength'):
            self[1][0][1][0].update_attribs({"maxlength":self.get_field_value('maxlength')})
        if self.get_field_value('required'):
            self[1][0][1][0].update_attribs({"required":"required"})
        if self.get_field_value('type'):
            self[1][0][1][0].update_attribs({"type":self.get_field_value('type')})
        if self.get_field_value('pattern'):
            self[1][0][1][0].update_attribs({"pattern":self.get_field_value('pattern')})
        if self.get_field_value('title'):
            self[1][0][1][0].update_attribs({"title":self.get_field_value('title')})
        if self.get_field_value('input_class'):
            input_class = self.get_field_value('input_class')
        else:
            input_class = ''

        if self.error_status and self.get_field_value('input_errored_class'):
            if input_class:
                input_class = input_class + ' ' + self.get_field_value('input_errored_class')
            else:
                input_class = self.get_field_value('input_errored_class')
        elif self.get_field_value('set_input_errored') and self.get_field_value('input_errored_class'):
            if input_class:
                input_class = input_class + ' ' + self.get_field_value('input_errored_class')
            else:
                input_class = self.get_field_value('input_errored_class')
        elif self.get_field_value('set_input_accepted') and self.get_field_value('input_accepted_class'):
            if input_class:
                input_class = input_class + ' ' + self.get_field_value('input_accepted_class')
            else:
                input_class = self.get_field_value('input_accepted_class')

        if input_class:
            self[1][0][1][0].update_attribs({"class":input_class})

        # set the label 'for' attribute
        self[1][0][0].update_attribs({'for':self[1][0][1][0].get_id()})

        # submit button
        if self.get_field_value('button_class'):
            self[1][0][1][1].update_attribs({"class": self.get_field_value('button_class')})
        if self.get_field_value('button_text'):
            self[1][0][1][1][0] = self.get_field_value('button_text')


        # add ident and four hidden fields
        self.add_hiddens(self[1], page)



    def _build_js(self, page, ident_list, environ, call_data, lang):
        """Sets a submit event handler"""
        jscript = """  $("#{ident} form").on("submit input", function(e) {{
    SKIPOLE.widgets["{ident}"].eventfunc(e);
    }});
""".format(ident=self.get_id())
        if self._jsonurl:
            return jscript + self._make_fieldvalues('input_accepted_class', 'input_errored_class', url=self._jsonurl)
        return jscript + self._make_fieldvalues('input_accepted_class', 'input_errored_class')


    @classmethod
    def description(cls):
        """Returns a text string to illustrate the widget"""
        return """
<div> <!-- with widget id and class widget_class -->
  <div>  <!-- div hidden when no error is displayed, with class set to error_class on error -->
    <p> <!-- error message appears in this paragraph --> </p>
  </div>
  <form role="form" method="post"> <!-- action attribute set to action field -->
    <div> <!-- class attribute set to inputdiv_class -->
      <label> <!-- with class set to label_class and content to label, for set to input text id -->
      </label>
      <span>  <!-- class attribute set to inputandbutton_class -->
          <input type="text" />  <!-- class set to input_class -->
            <!-- input text value set to input_text -->
          <button type="submit"> <!-- with class set to button_class -->
            <!-- button_text -->
          </button>
      </span>
    </div>
    <!-- hidden input fields -->                              
  </form>
</div>"""



class SubmitTextInput3(Widget):
    """Defines paragraphs followed by a form with a text input field"""

    # js_validators is a class attribute, True if javascript validation is enabled
    js_validators=True

    error_location = (0,0,0)

    arg_descriptions = {
                        'show_para1':FieldArg("boolean", True, jsonset=True),
                        'para_text':FieldArg("text", '', jsonset=True),
                        'show_para2':FieldArg("boolean", True, jsonset=True),
                        'textblock_ref':FieldArg("textblock_ref", ""),
                        'label':FieldArg("text", 'Your input:'),
                        'label_class':FieldArg("cssclass", ''),
                        'label_style':FieldArg("cssstyle", ''),
                        'button_label':FieldArg("text", 'Please submit:'),
                        'button_label_class':FieldArg("cssclass", ''),
                        'button_label_style':FieldArg("cssstyle", ''),
                        'action':FieldArg("url", ''),
                        'action_json':FieldArg("url", ''),
                        'hidden_field1':FieldArg("text", '', valdt=True),
                        'hidden_field2':FieldArg("text", '', valdt=True),
                        'hidden_field3':FieldArg("text", '', valdt=True),
                        'hidden_field4':FieldArg("text", '', valdt=True),
                        'target':FieldArg("text",''),
                        'button_text':FieldArg("text",'Submit'),
                        'button_class':FieldArg("cssclass", ''),
                        'buttondiv_class':FieldArg("cssclass", ''),
                        'outer_class':FieldArg("cssclass", ''),
                        'inputdiv_class':FieldArg("cssclass", ''),
                        'error_class':FieldArg("cssclass", ""),
                        'input_text':FieldArg("text", '', valdt=True, jsonset=True),
                        'size':FieldArg("text", ''),
                        'maxlength':FieldArg("text", ''),
                        'required':FieldArg("boolean", False),
                        'type':FieldArg("text", 'text'),
                        'pattern':FieldArg("text", ''),
                        'title':FieldArg("text", ''),
                        'input_accepted_class':FieldArg("cssclass", ''),
                        'input_errored_class':FieldArg("cssclass", ''),
                        'input_class':FieldArg("cssclass", ''),
                        'set_input_accepted':FieldArg("boolean", False, jsonset=True),
                        'set_input_errored':FieldArg("boolean", False, jsonset=True),
                        'hide':FieldArg("boolean", False, jsonset=True)
                       }

    def __init__(self, name=None, brief='', **field_args):
        """
        show_para1: True if the paragraph is to be visible, False if not
        para_text: The text appearing in the paragraph
        show_para2: True if the paragraph is to be visible, False if not
        textblock_ref: The reference of the TextBlock appearing in the second paragraph
        label: The text displayed to the left of the text input field
        label_class: The css class of the label
        button_label: The text displayed to the left of the button
        button_label_class: The css class of the label to the left of the button
        action_json:  if a value set, and client has jscript enabled, this is the page ident, label, url this button links to, expects a json page back
        action: The page ident, label, url this button links to
        hidden_field1: A hidden field value, leave blank if unused, name used as the get field name
        hidden_field2: A second hidden field value, leave blank if unused, name used as the get field name
        hidden_field3: A third hidden field value, leave blank if unused, name used as the get field name
        hidden_field4: A fourth hidden field value, leave blank if unused, name used as the get field name
        target: if given, the target attribute will be set
        button_text: The text on the button
        buttondiv_class: the class attribute of the div which contains the button
        button_class: the class attribute of the button
        outer_class: the class attribute of the outside div
        inputdiv_class: the class attribute of the tag which contains the label and input text
        error_class: The class applied to the paragraph containing the error message on error.
        input_text: The default text in the text input field, field name used as the name attribute
        size: The number of characters appearing in the text input area
        maxlength: The maximum number of characters accepted in the text area
        required: Set True to put the 'required' flag into the input field
        type: the type set on the input field, such as text or email
        pattern: regular expression pattern
        title: helps describe the pattern
        input_accepted_class: A class which can be set on the input field
        input_errored_class: A class which can be set on the input field
        input_class: Class set on the input field
        set_input_accepted: If True, input_accepted_class will be set on the input field
        set_errored_accepted: If True, input_errored_class will be set on the input field
        hide: If True, widget is hidden
        """
        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)

        # error div at 0
        self[0] = tag.Part(tag_name="div", attribs={"style":"display:none;"})
        self[0][0] = tag.Part(tag_name="p")
        self[0][0][0] = ''
        # div containing paragraphs
        self[1] = tag.Part(tag_name='div')
        # paragraph one and two
        self[1][0] = tag.Part(tag_name='p')
        self[1][0][0] =''
        self[1][1] = tag.Part(tag_name='p')
        self[1][1][0] = ''

        # The form
        self[2] = tag.Part(tag_name='form', attribs={"role":"form", "method":"post"})

        # div containing label and input text
        self[2][0] = tag.Part(tag_name='div')
        # the label
        self[2][0][0] = tag.Part(tag_name="label", hide_if_empty=True)
        # the text input field
        self[2][0][1] = tag.ClosedPart(tag_name="input", attribs ={"type":"text"})


        # div containing button
        self[2][1] = tag.Part(tag_name='div')
        # the label on the left of the button
        self[2][1][0] = tag.Part(tag_name="label", hide_if_empty=True)
        # the submit button
        self[2][1][1] = tag.ClosedPart(tag_name="input", attribs={"type":"submit"})
        self._jsonurl = ''


    def _build(self, page, ident_list, environ, call_data, lang):
        "build the form"
        if self.get_field_value("target"):
            self[2].update_attribs({"target":self.get_field_value("target")})
        # Hides widget if no error and hide is True
        self.widget_hide(self.get_field_value("hide"))
        self._jsonurl = skiboot.get_url(self.get_field_value("action_json"), proj_ident=page.proj_ident)
        if self.get_field_value('error_class'):
            self[0].update_attribs({"class":self.get_field_value('error_class')})
        if self.error_status:
            self[0].del_one_attrib("style")
        if not self.get_field_value("action"):
            # setting self._error replaces the entire tag
            self._error = "Warning: No form action"
            return
        actionurl = skiboot.get_url(self.get_field_value("action"),  proj_ident=page.proj_ident)
        if not actionurl:
            # setting self._error replaces the entire tag
            self._error = "Warning: broken link"
            return
        # update the action of the form
        self[2].update_attribs({"action": actionurl})
        # the class of the outer div
        if self.get_field_value('outer_class'):
            self[1].attribs = {"class": self.get_field_value('outer_class')}

        # set paragraphs
        if not self.get_field_value("show_para1"):
            self[1][0].attribs={"style":"display:none;"}
        self[1][0][0] = self.get_field_value("para_text")
        if not self.get_field_value("show_para2"):
            self[1][1].attribs={"style":"display:none;"}
        # define the textblock
        tblock = self.get_field_value("textblock_ref")
        if tblock:
            tblock.proj_ident = page.proj_ident
            self[1][1][0] = tblock

        # the div holding label and input text
        if self.get_field_value('inputdiv_class'):
            self[2][0].attribs = {"class": self.get_field_value('inputdiv_class')}

        if self.get_field_value('label'):
            self[2][0][0][0] = self.get_field_value('label')
        if self.get_field_value('label_class'):
            self[2][0][0].attribs = {"class": self.get_field_value('label_class')}
        if self.get_field_value('label_style'):
            self[2][0][0].attribs = {"style": self.get_field_value('label_style')}

        # set an id in the input field for the 'label for' tag
        self[2][0][1].insert_id()
        self[2][0][1].update_attribs({"name":self.get_formname('input_text'), "value":self.get_field_value('input_text')})
        if self.get_field_value('size'):
            self[2][0][1].update_attribs({"size":self.get_field_value('size')})
        if self.get_field_value('maxlength'):
            self[2][0][1].update_attribs({"maxlength":self.get_field_value('maxlength')})
        if self.get_field_value('required'):
            self[2][0][1].update_attribs({"required":"required"})
        if self.get_field_value('type'):
            self[2][0][1].update_attribs({"type":self.get_field_value('type')})
        if self.get_field_value('pattern'):
            self[2][0][1].update_attribs({"pattern":self.get_field_value('pattern')})
        if self.get_field_value('title'):
            self[2][0][1].update_attribs({"title":self.get_field_value('title')})

        if self.get_field_value('input_class'):
            input_class = self.get_field_value('input_class')
        else:
            input_class = ''

        if self.error_status and self.get_field_value('input_errored_class'):
            if input_class:
                input_class = input_class + ' ' + self.get_field_value('input_errored_class')
            else:
                input_class = self.get_field_value('input_errored_class')
        elif self.get_field_value('set_input_errored') and self.get_field_value('input_errored_class'):
            if input_class:
                input_class = input_class + ' ' + self.get_field_value('input_errored_class')
            else:
                input_class = self.get_field_value('input_errored_class')
        elif self.get_field_value('set_input_accepted') and self.get_field_value('input_accepted_class'):
            if input_class:
                input_class = input_class + ' ' + self.get_field_value('input_accepted_class')
            else:
                input_class = self.get_field_value('input_accepted_class')

        if input_class:
            self[2][0][1].update_attribs({"class":input_class})

        # set the label 'for' attribute
        self[2][0][0].update_attribs({'for':self[2][0][1].get_id()})

        # the div holding button
        if self.get_field_value('buttondiv_class'):
            self[2][1].attribs = {"class": self.get_field_value('buttondiv_class')}

        if self.get_field_value('button_label'):
            self[2][1][0][0] = self.get_field_value('button_label')
        if self.get_field_value('button_label_class'):
            self[2][1][0].attribs = {"class": self.get_field_value('button_label_class')}
        if self.get_field_value('button_label_style'):
            self[2][1][0].attribs = {"style": self.get_field_value('button_label_style')}


        # submit button
        self[2][1][1].update_attribs({"value":self.get_field_value('button_text')})
        # set an id in the submit button for the 'label for' tag
        self[2][1][1].insert_id()
        # the button class
        if self.get_field_value('button_class'):
            self[2][1][1].update_attribs({"class": self.get_field_value('button_class')})


        # set the label 'for' attribute
        self[2][1][0].update_attribs({'for':self[2][1][1].get_id()})

        # add ident and four hidden fields
        self.add_hiddens(self[2], page)



    def _build_js(self, page, ident_list, environ, call_data, lang):
        """Sets a submit event handler"""
        jscript = """  $("#{ident} form").on("submit input", function(e) {{
    SKIPOLE.widgets["{ident}"].eventfunc(e);
    }});
""".format(ident=self.get_id())
        if self._jsonurl:
            return jscript + self._make_fieldvalues('input_accepted_class', 'input_errored_class', url=self._jsonurl)
        return jscript + self._make_fieldvalues('input_accepted_class', 'input_errored_class')

    @classmethod
    def description(cls):
        """Returns a text string to illustrate the widget"""
        return """
<div> <!-- with widget id and class widget_class -->
  <div> <!-- normally hidden div, with class error_class -->
    <p> <!-- Any error text appears here --> </p>
  </div>
  <div> <!-- class attribute set to outer_class -->
    <p><!-- Paragraph 1, hidden if show_para1 is False -->
      <!-- Containing para_text -->
    </p>
    <p><!-- Paragraph 2, hidden if show_para2 is False -->
      <!-- Contains TextBlock -->
    </p>
  <div>
  <form method="post"> <!-- action attribute set to action field -->
    <div> <!-- class attribute set to inputdiv_class -->
      <label> <!-- with class set to label_class and content to label -->
      </label>
      <input type=\"text\" /> <!-- input text value set to input_text -->
    </div>
    <div> <!-- class attribute set to buttondiv_class -->
      <label> <!-- with class set to button_label_class and content to button_label -->
      </label>
      <input type=\"submit\" /> <!-- button value set to button_text and class to button_class -->
    </div>
    <!-- hidden input fields -->
  </form>    
</div>"""


class TwoInputsSubmit1(Widget):
    """Defines a form containing two text input fields."""

    error_location = (0,0,0)

    # js_validators is a class attribute, True if javascript validation is enabled
    js_validators=True

    arg_descriptions = {'action':FieldArg("url", ''),
                        'action_json':FieldArg("url", ''),
                        'error_class':FieldArg("cssclass", ""),
                        'inputdiv_class':FieldArg("cssclass", ''),
                        'label':FieldArg("text", 'Your input:'),
                        'label_class':FieldArg("cssclass", ''),
                        'label_style':FieldArg("cssstyle", ''),
                        'hidden_field1':FieldArg("text", '', valdt=True),
                        'hidden_field2':FieldArg("text", '', valdt=True),
                        'hidden_field3':FieldArg("text", '', valdt=True),
                        'hidden_field4':FieldArg("text", '', valdt=True),
                        'button_text':FieldArg("text", 'Submit'),
                        'button_class':FieldArg("cssclass", ''),
                        'input_text1':FieldArg("text", '', valdt=True, jsonset=True),
                        'size1':FieldArg("text", ''),
                        'maxlength1':FieldArg("text", ''),
                        'disabled1':FieldArg("boolean", False),
                        'required1':FieldArg("boolean", False),
                        'type1':FieldArg("text", 'text'),
                        'pattern1':FieldArg("text", ''),
                        'title1':FieldArg("text", ''),
                        'set_input_accepted1':FieldArg("boolean", False, jsonset=True),
                        'set_input_errored1':FieldArg("boolean", False, jsonset=True),
                        'input_text2':FieldArg("text", '', valdt=True, jsonset=True),
                        'size2':FieldArg("text", ''),
                        'maxlength2':FieldArg("text", ''),
                        'disabled2':FieldArg("boolean", False),
                        'required2':FieldArg("boolean", False),
                        'type2':FieldArg("text", 'text'),
                        'pattern2':FieldArg("text", ''),
                        'title2':FieldArg("text", ''),
                        'set_input_accepted2':FieldArg("boolean", False, jsonset=True),
                        'set_input_errored2':FieldArg("boolean", False, jsonset=True),
                        'input_class1':FieldArg("cssclass", ''),
                        'input_class2':FieldArg("cssclass", ''),
                        'input_accepted_class':FieldArg("cssclass", ''),
                        'input_errored_class':FieldArg("cssclass", '')
                       }


    def __init__(self, name=None, brief='', **field_args):
        """
        action: The page ident this button links to
        action_json:  if a value set, and client has jscript enabled, this is the page ident, label, url this button links to, expects a json page back
        error_class: The class applied to the paragraph containing the error message on error.
        inputdiv_class: the class attribute of the tag which contains the label and inputs
        label: The text displayed to the left of the text input field
        label_class: The css class of the label
        hidden_field1: A hidden field value, leave blank if unused, name used as the get field name
        hidden_field2: A second hidden field value, leave blank if unused, name used as the get field name
        hidden_field3: A third hidden field value, leave blank if unused, name used as the get field name
        hidden_field4: A fourth hidden field value, leave blank if unused, name used as the get field name
        button_text: The text on the button
        button_class: the class attribute of the button
        input_accepted_class: A class which can be set on each input field
        input_errored_class: A class which can be set on each input field
        input_text1: The default text in the first text input field, field name used as the name attribute
        input_class1: Class set on the first input field
        size1: The number of characters appearing in the first text input area
        maxlength1: The maximum number of characters accepted in the first text area
        disabled1: Set True if the first input field is to be disabled
        required1: Set True to put the 'required' flag into the first input field
        type1: the type set on the first input field, such as text or email
        pattern1: regular expression pattern for first input field
        title1: helps describe the pattern of the first input field
        input_text2: The default text in the second text input field, field name used as the name attribute
        input_class2: Class set on the second input field
        size2: The number of characters appearing in the second text input area
        maxlength2: The maximum number of characters accepted in the second text area
        disabled2: Set True if the second field is to be disabled
        required2: Set True to put the 'required' flag into the second input field
        type2: the type set on the second input field, such as text or email
        pattern2: regular expression pattern for the second input field
        title2: helps describe the pattern of the second input field
        """
        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
        # error div at 0
        self[0] = tag.Part(tag_name="div", attribs={"style":"display:none;"})
        self[0][0] = tag.Part(tag_name="p")
        self[0][0][0] = ''
        # The form
        self[1] = tag.Part(tag_name='form', attribs={"role":"form", "method":"post"})
        # div containing label, input text and button
        self[1][0] = tag.Part(tag_name='div')

        # the label
        self[1][0][0] = tag.Part(tag_name="label", hide_if_empty=True)
        # the first text input field
        self[1][0][1] = tag.ClosedPart(tag_name="input", attribs ={"type":"text"})
        # the second text input field
        self[1][0][2] = tag.ClosedPart(tag_name="input", attribs ={"type":"text"})
        # the submit button
        self[1][0][3] = tag.ClosedPart(tag_name="input", attribs={"type":"submit"})
        self._jsonurl = ''


    def _build(self, page, ident_list, environ, call_data, lang):
        "build the form"
        if not self.get_field_value("action"):
            # setting self._error replaces the entire tag
            self._error = "Warning: No form action"
            return
        actionurl = skiboot.get_url(self.get_field_value("action"),  proj_ident=page.proj_ident)
        if not actionurl:
            # setting self._error replaces the entire tag
            self._error = "Warning: broken link"
            return
        # update the action of the form
        self[1].update_attribs({"action": actionurl})
        self._jsonurl = skiboot.get_url(self.get_field_value("action_json"), proj_ident=page.proj_ident)
        if self.get_field_value('error_class'):
            self[0].update_attribs({"class":self.get_field_value('error_class')})
        if self.error_status:
            self[0].del_one_attrib("style")

        # the div holding label, input text and button
        if self.get_field_value('inputdiv_class'):
            self[1][0].attribs = {"class": self.get_field_value('inputdiv_class')}

        # set up label
        if self.get_field_value('label'):
            self[1][0][0][0] = self.get_field_value('label')
        if self.get_field_value('label_class'):
            self[1][0][0].attribs = {"class": self.get_field_value('label_class')}
        if self.get_field_value('label_style'):
            self[1][0][0].attribs = {"style": self.get_field_value('label_style')}
        # first input field
        self[1][0][1].update_attribs({"name":self.get_formname('input_text1'), "value":self.get_field_value('input_text1')})
        if self.get_field_value('size1'):
            self[1][0][1].update_attribs({"size":self.get_field_value('size1')})
        if self.get_field_value('maxlength1'):
            self[1][0][1].update_attribs({"maxlength":self.get_field_value('maxlength1')})
        if self.get_field_value('disabled1'):
            self[1][0][1].update_attribs({"disabled":"disabled"})
        if self.get_field_value('required1'):
            self[1][0][1].update_attribs({"required":"required"})
        if self.get_field_value('type1'):
            self[1][0][1].update_attribs({"type":self.get_field_value('type1')})
        if self.get_field_value('pattern1'):
            self[1][0][1].update_attribs({"pattern":self.get_field_value('pattern1')})
        if self.get_field_value('title1'):
            self[1][0][1].update_attribs({"title":self.get_field_value('title1')})

        if self.get_field_value('input_class1'):
            input_class1 = self.get_field_value('input_class1')
        else:
            input_class1 = ''

        if self.error_status and self.get_field_value('input_errored_class'):
            if input_class1:
                input_class1 = input_class1 + ' ' + self.get_field_value('input_errored_class')
            else:
                input_class1 = self.get_field_value('input_errored_class')
        elif self.get_field_value('set_input_errored1') and self.get_field_value('input_errored_class'):
            if input_class1:
                input_class1 = input_class1 + ' ' + self.get_field_value('input_errored_class')
            else:
                input_class1 = self.get_field_value('input_errored_class')
        elif self.get_field_value('set_input_accepted1') and self.get_field_value('input_accepted_class'):
            if input_class1:
                input_class1 = input_class1 + ' ' + self.get_field_value('input_accepted_class')
            else:
                input_class1 = self.get_field_value('input_accepted_class')

        if input_class1:
            self[1][0][1].update_attribs({"class":input_class1})


        # second input field
        self[1][0][2].update_attribs({"name":self.get_formname('input_text2'), "value":self.get_field_value('input_text2')})
        if self.get_field_value('size2'):
            self[1][0][2].update_attribs({"size":self.get_field_value('size2')})
        if self.get_field_value('maxlength2'):
            self[1][0][2].update_attribs({"maxlength":self.get_field_value('maxlength2')})
        if self.get_field_value('disabled2'):
            self[1][0][2].update_attribs({"disabled":"disabled"})
        if self.get_field_value('required2'):
            self[1][0][2].update_attribs({"required":"required"})
        if self.get_field_value('type2'):
            self[1][0][2].update_attribs({"type":self.get_field_value('type2')})
        if self.get_field_value('pattern2'):
            self[1][0][2].update_attribs({"pattern":self.get_field_value('pattern2')})
        if self.get_field_value('title2'):
            self[1][0][2].update_attribs({"title":self.get_field_value('title2')})

        if self.get_field_value('input_class2'):
            input_class2 = self.get_field_value('input_class2')
        else:
            input_class2 = ''

        if self.error_status and self.get_field_value('input_errored_class'):
            if input_class2:
                input_class2 = input_class2 + ' ' + self.get_field_value('input_errored_class')
            else:
                input_class2 = self.get_field_value('input_errored_class')
        elif self.get_field_value('set_input_errored2') and self.get_field_value('input_errored_class'):
            if input_class2:
                input_class2 = input_class2 + ' ' + self.get_field_value('input_errored_class')
            else:
                input_class2 = self.get_field_value('input_errored_class')
        elif self.get_field_value('set_input_accepted2') and self.get_field_value('input_accepted_class'):
            if input_class2:
                input_class2 = input_class2 + ' ' + self.get_field_value('input_accepted_class')
            else:
                input_class2 = self.get_field_value('input_accepted_class')

        if input_class2:
            self[1][0][2].update_attribs({"class":input_class2})

        # the submit button
        self[1][0][3].update_attribs({"value":self.get_field_value('button_text')})
        # the button class
        if self.get_field_value('button_class'):
            self[1][0][3].update_attribs({"class": self.get_field_value('button_class')})

        # set an id in the first input field for the 'label for' tag
        self[1][0][1].insert_id()
        # set the label 'for' attribute
        self[1][0][0].update_attribs({'for':self[1][0][1].get_id()})

        # add ident and four hidden fields
        self.add_hiddens(self[1], page)


    def _build_js(self, page, ident_list, environ, call_data, lang):
        """Sets a submit event handler"""
        jscript = """  $("#{ident} form").on("submit input", function(e) {{
    SKIPOLE.widgets["{ident}"].eventfunc(e);
    }});
""".format(ident=self.get_id())
        if self._jsonurl:
            return jscript + self._make_fieldvalues('input_accepted_class', 'input_errored_class', url=self._jsonurl)
        return jscript + self._make_fieldvalues('input_accepted_class', 'input_errored_class')


    @classmethod
    def description(cls):
        """Returns a text string to illustrate the widget"""
        return """
<div> <!-- with widget id and class widget_class -->
  <div> <!-- normally hidden div, with class error_class -->
    <p> <!-- Any error text appears here --> </p>
  </div>
  <form method="post"> <!-- action attribute set to action field -->
    <div> <!-- class attribute set to inputdiv_class -->
      <label> <!-- with class set to label_class and content to label -->
      </label>
      <input type=\"text\" /> <!-- input text value set to input_text1, class to input_class1 -->
      <input type=\"text\" /> <!-- input text value set to input_text2, class to input_class2 -->
      <input type=\"submit\" /> <!-- button value set to button_text and class to button_class -->
    </div>
    <!-- hidden input fields -->    
  </form>
</div>"""


class SubmitDict1(Widget):
    """Defines a form with a list of input fields followed by a single submit button
       to allow a client to input a dictionary"""

    display_errors = False

    arg_descriptions = {'action':FieldArg("url", ''),
                        'button_text':FieldArg("text", "Submit"),
                        'button_class':FieldArg("cssclass", ''),
                        'input_dict':FieldArgDict('text', valdt=False, senddict=True),
                        'input_class':FieldArg("cssclass", ''),
                        'inputdiv_class':FieldArg("cssclass", ''),
                        'inputdiv_style':FieldArg("cssstyle", ''),
                        'ul_class':FieldArg("cssclass", ""),
                        'ul_style':FieldArg("cssstyle", ""),
                        'li_class':FieldArg("cssclass", ""),
                        'li_style':FieldArg("cssstyle", ""),
                        'hidden_field1':FieldArg("text", '', valdt=True),
                        'hidden_field2':FieldArg("text", '', valdt=True),
                        'hidden_field3':FieldArg("text", '', valdt=True),
                        'hidden_field4':FieldArg("text", '', valdt=True)
                       }

    def __init__(self, name=None, brief='', **field_args):
        """
        action: The label or ident of the action page
        button_text: text appearing on the button
        button_class: The css class of the button
        input_dict: an ordered dictionary, keys used in the field name, values are the text input
        input_class: Class set on each input field
        ul_class: Unordered list class
        ul_style: Style applied to ul element
        li_class: list element class
        li_style: Style applied to li element
        hidden_field1: A hidden field value, leave blank if unused, name used as the get field name
        hidden_field2: A second hidden field value, leave blank if unused, name used as the get field name
        hidden_field3: A third hidden field value, leave blank if unused, name used as the get field name
        hidden_field4: A fourth hidden field value, leave blank if unused, name used as the get field name
        """
        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
        # The form
        self[0] = tag.Part(tag_name='form', attribs={"method":"post"})
        self[0][0] = tag.Part(tag_name='ul')
        # the submit button in a div
        self[0][1] = tag.Part(tag_name="div")
        self[0][1][0] = tag.ClosedPart(tag_name="input")

    def _build(self, page, ident_list, environ, call_data, lang):
        "Build the form and list"

        input_dict = self.get_field_value("input_dict")
        if not input_dict:
            self.show = False
            return

        # get the form action url
        if not self.get_field_value("action"):
            # setting self._error replaces the entire tag
            self._error = "Warning: No form action"
            return
        action_url = skiboot.get_url(self.get_field_value("action"), proj_ident=page.proj_ident)
        if not action_url:
            # setting self._error replaces the entire tag by the self._error message
            self._error = "Warning: broken link"
            return
        self[0].update_attribs({"action": action_url})


        if self.get_field_value("ul_class"):
            self[0][0].update_attribs({"class": self.get_field_value("ul_class")})
        if self.get_field_value("ul_style"):
            self[0][0].update_attribs({"style": self.get_field_value("ul_style")})

        li_class =  self.get_field_value("li_class")

        input_name = self.get_formname("input_dict") + '-'

        if self.get_field_value('input_class'):
            input_class = self.get_field_value('input_class')
        else:
            input_class = ''

        linumber = -1
        for key, value in input_dict.items():
            keyed_name = input_name + key
            if input_class:
                li_input = tag.ClosedPart(tag_name="input",
                                          attribs ={"name":keyed_name,
                                                    "value":value,
                                                    "type":"text",
                                                    "class":input_class})
            else:
                li_input = tag.ClosedPart(tag_name="input",
                                          attribs ={"name":keyed_name,
                                                    "value":value,
                                                    "type":"text"})
            linumber += 1
            if li_class:
                self[0][0][linumber] = tag.Part(tag_name="li", attribs ={"class":li_class})
            else:
                self[0][0][linumber] = tag.Part(tag_name="li")
            if self.get_field_value("li_style"):
                self[0][0][linumber].update_attribs({"style": self.get_field_value("li_style")})


            self[0][0][linumber][0] = li_input

        # list done, now for submit button

        # the div holding button
        if self.get_field_value('inputdiv_class') and self.get_field_value('inputdiv_style'):
            self[0][1].attribs = {"class" : self.get_field_value('inputdiv_class'), "style": self.get_field_value('inputdiv_style')}
        elif self.get_field_value('inputdiv_class'):
            self[0][1].attribs = {"class" : self.get_field_value('inputdiv_class')}
        elif self.get_field_value('inputdiv_style'):
            self[0][1].attribs = {"style" : self.get_field_value('inputdiv_style')}

        if self.get_field_value('button_class'):
            self[0][1][0].attribs = {"value":self.get_field_value('button_text'), "type":"submit", "class": self.get_field_value('button_class')}
        else:
            self[0][1][0].attribs = {"value":self.get_field_value('button_text'), "type":"submit"}

        # add ident and four hidden fields
        self.add_hiddens(self[0], page)


    @classmethod
    def description(cls):
        """Returns a text string to illustrate the widget"""
        return """
<div> <!-- with widget id and class widget_class -->
  <form method=\"post\"> <!-- action attribute set to action field -->
    <ul> <!-- with class set to ul_class and style to ul_style -->
      <!-- a li is created for each value in the input_dict dictionary -->
      <li>  <!-- with class set to li_class and style to li_style -->
        <input type="text" /> <!-- with CSS class input_class -->
             <!-- and with input name 'widgetname:input_dict-keyname' -->
      </li>
    </ul>
    <div> <!-- class attribute set to inputdiv_class, style to inputdiv_style -->
      <input type="submit" /> <!-- with value button_text, and CSS class button_class -->
    </div>
    <!-- hidden fields -->                              
  </form>
</div>"""







from skipole import ValidateError, FailPage, ServerError, GoTo

from skipole.skilift import get_itemnumber
from skipole.skilift.editpage import rename_page, page_description, new_parent



def submit_rename_page(skicall):
    "rename this page"

    call_data = skicall.call_data
    page_data = skicall.page_data

    if 'page_number' not in call_data:
        raise FailPage(message = "Page missing")
    if 'new_name' not in call_data:
        raise FailPage(message="No new page name has been given")
    # Rename the page
    # call skilift.editpage.rename_page which returns a new pchange
    try:
        call_data['pchange'] = rename_page(call_data['editedprojname'], call_data['page_number'], call_data['pchange'], call_data['new_name'])
    except ServerError as e:
        raise FailPage(e.message)
    page_data['page_edit','p_rename','set_input_accepted'] = True
    call_data['status'] = 'Page renamed'



def submit_new_parent(skicall):
    "Gives a page a new parent"

    call_data = skicall.call_data
    page_data = skicall.page_data

    project = call_data['editedprojname']
    pagenumber = call_data['page_number']
    pchange = call_data['pchange']
    if not 'parent_ident' in call_data:
        raise FailPage(message="No parent ident given")
    parent_ident = call_data['parent_ident']
    new_parent_number = get_itemnumber(project, parent_ident)
    if new_parent_number is None:
        raise FailPage(message="No valid parent ident given")
    try:
        call_data['pchange'] = new_parent(project, pagenumber, pchange, new_parent_number)
    except ServerError as e:
        raise FailPage(message=e.message)
    call_data['status'] = 'Page moved'


def submit_page_brief(skicall):
    "Sets new page brief"

    call_data = skicall.call_data
    page_data = skicall.page_data

    project = call_data['editedprojname']
    pagenumber = call_data['page_number']
    pchange = call_data['pchange']
    if not 'page_brief' in call_data:
        raise FailPage(message="No page brief given", widget="p_brief")
    new_brief = call_data['page_brief']
    if not new_brief:
        raise FailPage(message="No page brief given", widget="p_brief")
    # call skilift.editpage.page_description which returns a new pchange
    try:
        call_data['pchange'] = page_description(project, pagenumber, pchange, new_brief)
    except ServerError as e:
        raise FailPage(e.message)
    call_data['status'] = 'Page brief set: %s' % (new_brief,)


def goto_edit(skicall):

    call_data = skicall.call_data
    page_data = skicall.page_data


    caller_num = skicall.caller_ident[1]
    if caller_num == 28009:
        raise GoTo(28007)         # css edit page
    elif caller_num == 20409:
        raise GoTo(20407)         # json edit page
    elif caller_num == 23209:
        raise GoTo(23207)          # retrieve template edit page contents
    elif caller_num == 23409:
        raise GoTo(23407)          # svg edit page
    elif caller_num == 26009:
        raise GoTo(26007)          # retrieve responder edit page contents
    elif caller_num == 29009:
        raise GoTo(29007)          # filepage edit page
    else:
        raise GoTo('admin_home', clear_page_data=True)





"""A Respond object instance is placed in a Respond page,
   The instance is callable, and the respondpage calls it
   to provide the page action"""


import re

from .. import skiboot, tag
from ..excepts import ValidateError, ServerError

from . import Respond


class CaseSwitch(Respond):
    """
Checks a single submitted field content against a list, and if a match occurs, passes the call
to an associated page (or responder) given by the value, which can be an ident, label or url.
If there is no match, the page (or responder) specified by 'alternate_ident' will be called.
This is not considered an error, and no error message will be raised
"""

    # This indicates a widgfield is required
    widgfield_required = True

    # This indicates a fail page ident is required
    alternate_ident_required = True

    # Options for the fields argument
    field_options = {'fields': True,                  # If False, no fields are expected
                     'widgfields':False,               # If True, fields are widgfields, if False, can be other constants
                     'widgfield_values':False,        # If True the field values are widgfields
                     'fields_optional': False,        # if fields is True, then False here means fields must be supplied
                     'field_values': True,            # if True, field values are used
                     'empty_values_allowed':False,     # If True, '' is a valid value, if False, some data must be provided
                     'single_field': False}            # Multiple fields accepted


    def _respond(self, skicall, form_data, caller_page, ident_list, proj_ident, rawformdata):
        "Matches the value given in field self.widgfield against the fields given"
        if (not self.widgfield.w) or (not self.widgfield.f):
            raise ServerError(message="Invalid widgfield set in CaseSwitch Responder")
        if self.widgfield in form_data:
            value = form_data[self.widgfield]
            if value in self.fields:
                return self.get_page_from_ident(self.fields[value], skicall, form_data, caller_page, ident_list, proj_ident, rawformdata)
        return self.get_alternate_page(skicall, form_data, caller_page, ident_list, proj_ident, rawformdata)

 
class EmptyGoto(Respond):
    """
For the given widgfield, if the submitted form has that widgfield with an empty value or the
widgfield is not present, then passes the call to target_ident, otherwise passes the call
to alternate_ident.
If no caller page ident is given then it calls the project validate error page.
It also takes a list of allowed page idents which are the possible callers to this page, if
empty, any page can call it - on failure, calls the project validate error page.
"""

    # This indicates a widgfield is required
    widgfield_required = True

    # This indicates a target page ident is required
    target_ident_required = True

    # This indicates an alternate page ident is required
    alternate_ident_required = True

    # This indicates a list of allowed caller idents is required
    allowed_callers_required = True

    # Options for the fields argument
    field_options = {'fields': False,                  # If False, no fields are expected
                     'widgfields':False,               # If True, fields are widgfields, if False, can be other constants
                     'widgfield_values':False,        # If True the field values are widgfields
                     'fields_optional': False,        # if fields is True, then False here means fields must be supplied
                     'field_values': False,            # if True, field values are used
                     'empty_values_allowed':False,     # If True, '' is a valid value, if False, some data must be provided
                     'single_field': False}            # Multiple fields accepted


    def _respond(self, skicall, form_data, caller_page, ident_list, proj_ident, rawformdata):
        "Matches the field values against the data"
        if caller_page is None:
            raise ValidateError()
        self._check_allowed_callers(skicall, form_data, caller_page, ident_list, proj_ident, rawformdata)
        if (self.widgfield not in form_data) or (form_data[self.widgfield] == ''):
            return self.get_target_page(skicall, form_data, caller_page, ident_list, proj_ident, rawformdata)
        return self.get_alternate_page(skicall, form_data, caller_page, ident_list, proj_ident, rawformdata)


class EmptyCallDataGoto(Respond):
    """
For a single given key in call data, if call_data has that key with an empty value or the
key is not present, then passes the call to target_ident, otherwise passes the call
to alternate_ident.
"""

    # This indicates a target page ident is required
    target_ident_required = True

    # This indicates an alternate page ident is required
    alternate_ident_required = True

    # Options for the fields argument
    field_options = {'fields': True,                  # If False, no fields are expected
                     'widgfields':False,               # If True, fields are widgfields, if False, can be other constants
                     'widgfield_values':False,        # If True the field values are widgfields
                     'fields_optional': False,        # if fields is True, then False here means fields must be supplied
                     'field_values': False,            # if True, field values are used
                     'empty_values_allowed':True,     # If True, '' is a valid value, if False, some data must be provided
                     'single_field': True}            # Only a single field is accepted


    def _respond(self, skicall, form_data, caller_page, ident_list, proj_ident, rawformdata):
        "Matches the field against the data"
        call_data = skicall.call_data
        for field in self.fields:
            if (field not in call_data) or (call_data[field] == ''):
                return self.get_target_page(skicall, form_data, caller_page, ident_list, proj_ident, rawformdata)
        return self.get_alternate_page(skicall, form_data, caller_page, ident_list, proj_ident, rawformdata)



class DelCallDataItem(Respond):
    """
For a single given key in call data, if present, that key:value will be deleted from call_data.
"""

    # This indicates a target page ident is required
    target_ident_required = True


    # Options for the fields argument
    field_options = {'fields': True,                  # If False, no fields are expected
                     'widgfields':False,               # If True, fields are widgfields, if False, can be other constants
                     'widgfield_values':False,        # If True the field values are widgfields
                     'fields_optional': False,        # if fields is True, then False here means fields must be supplied
                     'field_values': False,            # if True, field values are used
                     'empty_values_allowed':True,     # If True, '' is a valid value, if False, some data must be provided
                     'single_field': True}            # Only a single field is accepted


    def _respond(self, skicall, form_data, caller_page, ident_list, proj_ident, rawformdata):
        "Deletes call_data item with the given field key"
        call_data = skicall.call_data
        for field in self.fields:
            if field in call_data:
                del call_data[field]
                # there should only be one field
                break
        return self.get_target_page(skicall, form_data, caller_page, ident_list, proj_ident, rawformdata)


class NoOperation(Respond):
    """
Goes to Target page, can be used as a temporary place holder
"""

    # This indicates a target page ident is required
    target_ident_required = True

    # Options for the fields argument
    field_options = {'fields': False,                  # If False, no fields are expected
                     'widgfields':False,               # If True, fields are widgfields, if False, can be other constants
                     'widgfield_values':False,        # If True the field values are widgfields
                     'fields_optional': False,        # if fields is True, then False here means fields must be supplied
                     'field_values': False,            # if True, field values are used
                     'empty_values_allowed':True,     # If True, '' is a valid value, if False, some data must be provided
                     'single_field': False}           # Multiple fields accepted


    def _respond(self, skicall, form_data, caller_page, ident_list, proj_ident, rawformdata):
        "Goes to target page"
        return self.get_target_page(skicall, form_data, caller_page, ident_list, proj_ident, rawformdata)








"""Functions for editing a widget"""

import pkgutil, importlib, inspect, re

from ..ski import widgets, dump_project
from ..ski.excepts import ServerError

from . import widget_info, fromjson, insert_item_in_page, insert_item_in_section, get_proj_page, get_proj_section


from .info_tuple import WidgetDescription, FieldDescription, ContainerInfo


# a search for anything none-alphanumeric and not an underscore
_AN = re.compile('[^\w]')


def widget_modules():
    "Return a tuple of widget module names"
    return tuple(name for (module_loader, name, ispkg) in pkgutil.iter_modules(widgets.__path__))


def widgets_in_module(module_name):
    "Returns a list of WidgetDescription's present in the module"
    module = importlib.import_module("skipole.ski.widgets." + module_name)
    widget_list = []
    for classname,obj in inspect.getmembers(module, lambda member: inspect.isclass(member) and (member.__module__ == module.__name__)):
        fields_single = [ FieldDescription(*fld) for fld in obj.field_arguments_single() ]
        fields_list = [ FieldDescription(*fld, False, False) for fld in obj.field_arguments_list() ]
        fields_table = [ FieldDescription(*fld, False, False) for fld in obj.field_arguments_table() ]
        fields_dictionary = [ FieldDescription(*fld, False, False) for fld in obj.field_arguments_dictionary() ]
        widget_list.append( WidgetDescription( module_name,
                                               classname,
                                               '',
                                               obj.field_arguments(),
                                               obj.len_containers(),
                                               obj.description(),
                                               fields_single,
                                               fields_list,
                                               fields_table,
                                               fields_dictionary,
                                               '',
                                               None
                                               ) )
    return widget_list


def _create_new_widget(project, module_name, widget_classname, name, brief):
    "Creates a new widget"
    if not name:
        raise ServerError("Invalid name")
    lower_name = name.lower()
    if (lower_name == 'body') or (lower_name == 'head') or (lower_name == 'svg')  or (lower_name == 'show_error'):
        raise ServerError(message="Unable to create the widget, the name given is reserved")
    if _AN.search(name):
        raise ServerError(message="Invalid name, alphanumeric and underscore only")
    if name[0] == '_':
        raise ServerError(message="Invalid name, must not start with an underscore")
    if name.isdigit():
        raise ServerError(message="Unable to create the widget, the name must include some letters")
    modules_tuple = widget_modules()
    if module_name not in modules_tuple:
        raise FailPage("Module not identified")
    module = importlib.import_module("skipole.ski.widgets." + module_name)
    widget_dict = {name:cls for (name,cls) in inspect.getmembers(module, lambda member: inspect.isclass(member) and (member.__module__ == module.__name__))}
    if widget_classname not in widget_dict:
        raise ServerError(message="Widget not identified")

    widget_cls = widget_dict[widget_classname]
    widget_instance = widget_cls(name=name, brief=brief)
    # set widget css class defaults, taken from defaults.json
    widget_fields = widget_instance.fields
    try:
        for fieldarg, field in widget_fields.items():
            if field.cssclass or field.cssstyle:
                # get default
                default_value = fromjson.get_widget_default_field_value(project, module_name, widget_classname, fieldarg)
                widget_instance.set_field_value(fieldarg, default_value)
    except e:
        raise ServerError("Unable to obtain defaults from defaults.json")
    return widget_instance


def create_new_widget_in_page(project, pagenumber, pchange, location, module_name, widget_classname, name, brief):
    "Creates a new widget in the given page, returns the new pchange and new location"
    widget_instance = _create_new_widget(project, module_name, widget_classname, name, brief)
    # call skilift.insert_item_in_page to insert the item, save the page and return pchange and new location
    return insert_item_in_page(project, pagenumber, pchange, location, widget_instance)


def create_new_widget_in_section(project, section_name, schange, location, module_name, widget_classname, name, brief):
    "Creates a new widget in the given section, returns the new schange and new location"
    widget_instance = _create_new_widget(project, module_name, widget_classname, name, brief)
    # call skilift.insert_item_in_section to insert the item, save the section and return schange and new location
    return insert_item_in_section(project, section_name, schange, location, widget_instance)


def page_widget(project, pagenumber, pchange, name):
    """Return a widget dictionary from the page"""
    proj, page = get_proj_page(project, pagenumber, pchange)
    widget = page.widgets.get(name)
    if (not isinstance(widget, widgets.Widget)) and (not isinstance(widget, widgets.ClosedWidget)):
        raise ServerError("Widget not found")
    parttext, partdict = widget.outline(project)
    return partdict


def section_widget(project, section_name, schange, name):
    """Return a widget dictionary from the section"""
    proj, section = get_proj_section(project, section_name, schange)
    widget = section.widgets.get(name)
    if (not isinstance(widget, widgets.Widget)) and (not isinstance(widget, widgets.ClosedWidget)):
        raise ServerError("Widget not found")
    parttext, partdict = widget.outline(project)
    return partdict


def page_widget_description(project, pagenumber, pchange, name):
    """Return a WidgetDescription from the page"""
    proj, page = get_proj_page(project, pagenumber, pchange)
    widget = page.widgets.get(name)
    if (not isinstance(widget, widgets.Widget)) and (not isinstance(widget, widgets.ClosedWidget)):
        raise ServerError("Widget not found")
    fields_single = [ FieldDescription(*fld) for fld in widget.field_arguments_single() ]
    fields_list = [ FieldDescription(*fld, False, False) for fld in widget.field_arguments_list() ]
    fields_table = [ FieldDescription(*fld, False, False) for fld in widget.field_arguments_table() ]
    fields_dictionary = [ FieldDescription(*fld, False, False) for fld in widget.field_arguments_dictionary() ]
    widgetsection_name, parent_widget_name, parent_container = widget.embedded
    return WidgetDescription( widget.module_name(),
                              widget.__class__.__name__,
                              widget.brief,
                              widget.field_arguments(),
                              widget.len_containers(),
                              widget.description(),
                              fields_single,
                              fields_list,
                              fields_table,
                              fields_dictionary,
                              parent_widget_name,
                              parent_container
                               )


def section_widget_description(project, section_name, schange, name):
    """Return a widget WidgetDescription from the section"""
    proj, section = get_proj_section(project, section_name, schange)
    widget = section.widgets.get(name)
    if (not isinstance(widget, widgets.Widget)) and (not isinstance(widget, widgets.ClosedWidget)):
        raise ServerError("Widget not found")
    fields_single = [ FieldDescription(*fld) for fld in widget.field_arguments_single() ]
    fields_list = [ FieldDescription(*fld, False, False) for fld in widget.field_arguments_list() ]
    fields_table = [ FieldDescription(*fld, False, False) for fld in widget.field_arguments_table() ]
    fields_dictionary = [ FieldDescription(*fld, False, False) for fld in widget.field_arguments_dictionary() ]
    widgetsection_name, parent_widget_name, parent_container = widget.embedded
    return WidgetDescription( widget.module_name(),
                              widget.__class__.__name__,
                              widget.brief,
                              widget.field_arguments(),
                              widget.len_containers(),
                              widget.description(),
                              fields_single,
                              fields_list,
                              fields_table,
                              fields_dictionary,
                              parent_widget_name,
                              parent_container
                               )


def rename_page_widget(project, pagenumber, pchange, name, new_name):
    """Given a widget at project, pagenumber, with name 
       sets a new name, returns page change uuid """
    if not new_name:
        raise ServerError("Invalid name")
    proj, page = get_proj_page(project, pagenumber, pchange)
    widget = page.widgets.get(name)
    if (not isinstance(widget, widgets.Widget)) and (not isinstance(widget, widgets.ClosedWidget)):
        raise ServerError("Widget not found")
    if name == new_name:
        raise ServerError("Name unchanged")
    new_lower_name = new_name.lower()
    if (new_lower_name == 'body') or (new_lower_name == 'head') or (new_lower_name == 'svg')  or (new_lower_name == 'show_error'):
        raise ServerError(message="Unable to create the widget, the name given is reserved")
    if _AN.search(new_name):
        raise ServerError(message="Invalid name, alphanumeric and underscore only")
    if new_name[0] == '_':
        raise ServerError(message="Invalid name, must not start with an underscore")
    if new_name.isdigit():
        raise ServerError(message="Unable to create the widget, the name must include some letters")
    if new_name in page.widgets:
        raise ServerError("Duplicate widget name in the page")
    if new_name in page.section_places:
        raise ServerError("This name clashes with a section alias within this page")
    widget.name = new_name
    # save the altered page, and return the page.change uuid
    return proj.save_page(page)


def new_brief_in_page_widget(project, pagenumber, pchange, name, brief):
    """Given a widget at project, pagenumber, with name 
       sets a new brief, returns page change uuid """
    proj, page = get_proj_page(project, pagenumber, pchange)
    widget = page.widgets.get(name)
    if (not isinstance(widget, widgets.Widget)) and (not isinstance(widget, widgets.ClosedWidget)):
        raise ServerError("Widget not found")
    widget.brief = brief
    # save the altered page, and return the page.change uuid
    return proj.save_page(page)


def rename_section_widget(project, section_name, schange, name, new_name):
    """Given a widget at project, pagenumber, with name 
       sets a new name, returns page change uuid """
    if not new_name:
        raise ServerError("Invalid name")
    proj, section = get_proj_section(project, section_name, schange)
    widget = section.widgets.get(name)
    if (not isinstance(widget, widgets.Widget)) and (not isinstance(widget, widgets.ClosedWidget)):
        raise ServerError("Widget not found")
    if name == new_name:
        raise ServerError("Name unchanged")
    new_lower_name = new_name.lower()
    if (new_lower_name == 'body') or (new_lower_name == 'head') or (new_lower_name == 'svg')  or (new_lower_name == 'show_error'):
        raise ServerError(message="Unable to create the widget, the name given is reserved")
    if _AN.search(new_name):
        raise ServerError(message="Invalid name, alphanumeric and underscore only")
    if new_name[0] == '_':
        raise ServerError(message="Invalid name, must not start with an underscore")
    if new_name.isdigit():
        raise ServerError(message="Unable to create the widget, the name must include some letters")
    if new_name in section.widgets:
        raise ServerError("Duplicate widget name in the section")
    widget.name = new_name
    # save the altered section, and return the section.change uuid
    return proj.add_section(section_name, section)


def new_brief_in_section_widget(project, section_name, schange, name, brief):
    """Given a widget at project, section_name, with name 
       sets a new brief, returns section change uuid """
    proj, section = get_proj_section(project, section_name, schange)
    widget = section.widgets.get(name)
    if (not isinstance(widget, widgets.Widget)) and (not isinstance(widget, widgets.ClosedWidget)):
        raise ServerError("Widget not found")
    widget.brief = brief
    # save the altered section, and return the section.change uuid
    return proj.add_section(section_name, section)


def set_widget_field_name_in_page(project, pagenumber, pchange, widget_name, field_arg, name):
    "Given a widget field, sets its name"
    proj, page = get_proj_page(project, pagenumber, pchange)
    widget = page.widgets.get(widget_name)
    if (not isinstance(widget, widgets.Widget)) and (not isinstance(widget, widgets.ClosedWidget)):
        raise ServerError("Widget not found")
    if not name:
        raise ServerError("Invalid name")
    if name.startswith('_'):
        raise ServerError(message="Invalid name - cannot start with an underscore.")
    if (name == "show_error") and (field_arg != "show_error"):
        raise ServerError(message="Invalid name - show_error is a reserved name.")
    try:
        widget.set_name(field_arg, name)
    except ValidateError as e:
        raise ServerError(message=e.message)
    # save the altered page, and return the page.change uuid
    return proj.save_page(page)


def set_widget_field_name_in_section(project, section_name, schange, widget_name, field_arg, name):
    "Given a widget field, sets its name"
    proj, section = get_proj_section(project, section_name, schange)
    widget = section.widgets.get(widget_name)
    if (not isinstance(widget, widgets.Widget)) and (not isinstance(widget, widgets.ClosedWidget)):
        raise ServerError("Widget not found")
    if not name:
        raise ServerError("Invalid name")
    if name.startswith('_'):
        raise ServerError(message="Invalid name - cannot start with an underscore.")
    if (name == "show_error") and (field_arg != "show_error"):
        raise ServerError(message="Invalid name - show_error is a reserved name.")
    try:
        widget.set_name(field_arg, name)
    except ValidateError as e:
        raise ServerError(message=e.message)
    # save the altered page, and return the page.change uuid
    return proj.add_section(section_name, section)


def set_widget_field_value_in_page(project, pagenumber, pchange, widget_name, field_arg, value):
    "Given a widget field, sets its value"
    proj, page = get_proj_page(project, pagenumber, pchange)
    widget = page.widgets.get(widget_name)
    if (not isinstance(widget, widgets.Widget)) and (not isinstance(widget, widgets.ClosedWidget)):
        raise ServerError("Widget not found")
    # only single values can be set, not lists tables or dictionaries
    single_field_args = { arg[0]: arg for arg in widget.field_arguments_single() }
    if field_arg not in single_field_args:
        raise ServerError("Cannot set a value on this field")
    field_info = single_field_args[field_arg]
    str_value = str(value)
    if (field_info[2] == "ident" or field_info[2] == "url") and str_value.isdigit():
        # The user is inputting a digit as a page ident
        str_value = project + '_' + str_value
    try:
        widget.set_field_value(field_arg, str_value)
    except ValidateError as e:
        raise ServerError(message=e.message)
    # save the altered page, and return the page.change uuid
    return proj.save_page(page)


def set_widget_field_value_in_section(project, section_name, schange, widget_name, field_arg, value):
    "Given a widget field, sets its value"
    proj, section = get_proj_section(project, section_name, schange)
    widget = section.widgets.get(widget_name)
    if (not isinstance(widget, widgets.Widget)) and (not isinstance(widget, widgets.ClosedWidget)):
        raise ServerError("Widget not found")
    # only single values can be set, not lists tables or dictionaries
    single_field_args = { arg[0]: arg for arg in widget.field_arguments_single() }
    if field_arg not in single_field_args:
        raise ServerError("Cannot set a value on this field")
    field_info = single_field_args[field_arg]
    str_value = str(value)
    if (field_info[2] == "ident" or field_info[2] == "url") and str_value.isdigit():
        # The user is inputting a digit as a page ident
        str_value = project + '_' + str_value
    try:
        widget.set_field_value(field_arg, str_value)
    except ValidateError as e:
        raise ServerError(message=e.message)
    # save the altered section, and return the section.change uuid
    return proj.add_section(section_name, section)


def container_in_page(project, pagenumber, pchange, widget_name, container):
    "Returns a ContainerInfo named tuple"
    proj, page = get_proj_page(project, pagenumber, pchange)
    widget = page.widgets.get(widget_name)
    if (not isinstance(widget, widgets.Widget)) and (not isinstance(widget, widgets.ClosedWidget)):
        raise ServerError("Widget not found")
    return ContainerInfo(container, widget.is_container_empty(container))


def container_in_section(project, section_name, schange, widget_name, container):
    "Returns a ContainerInfo named tuple"
    proj, section = get_proj_section(project, section_name, schange)
    widget = section.widgets.get(widget_name)
    if (not isinstance(widget, widgets.Widget)) and (not isinstance(widget, widgets.ClosedWidget)):
        raise ServerError("Widget not found")
    return ContainerInfo(container, widget.is_container_empty(container))









"""Defines widgets to aid in debugging"""

from string import Template
import pprint

from .. import skiboot
from .. import tag
from . import Widget, ClosedWidget, FieldArg, FieldArgList, FieldArgTable, FieldArgDict


class ShowEnviron(Widget):
    """This class is a div, followed by a paragraph of toptext, and then
       a <pre> tag with text content showing the environ dictionary.
       On error a paragraph above the toptext paragraph appears with error_class
    """

    error_location = (0,0,0)

    arg_descriptions = {'toptext':FieldArg("text", 'DEBUG MODE IS ON'),
                        'para_class':FieldArg("cssclass",""),
                        'error_class':FieldArg("cssclass", ""),
                        'show':FieldArg("boolean", True)  # set here as this value has a special textblock description
                       }

    def __init__(self, name=None, brief='', **field_args):
        """toptext: The text appearing above environ data
           error_class: If given the class of the error text"""
        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
        self[0] = tag.Part(tag_name="div", attribs={"style":"display:none;"})
        self[0][0] = tag.Part(tag_name="p")
        self[0][0][0] = ''
        self[1] = tag.Part(tag_name="p", hide_if_empty=True)
        self[2] = tag.Part(tag_name="pre")

    # special widget, not shown if debug is off

    def update(self, page, ident_list, environ, call_data, lang, ident_string, placename, embedded):
        """Sets self.show False if debug is False"""
        if not skiboot.get_debug():
            self.show = False
            return
        Widget.update(self, page, ident_list, environ, call_data, lang, ident_string, placename, embedded)

    def _build(self, page, ident_list, environ, call_data, lang):
        if self.get_field_value('error_class'):
            self[0].update_attribs({"class":self.get_field_value('error_class')})
        if self.error_status:
            self[0].del_one_attrib("style")
        if self.get_field_value('para_class'):
            self[1].update_attribs({"class":self.get_field_value('para_class')})
        self[1].text = self.get_field_value('toptext')
        self[2].text = pprint.pformat(environ)


    @classmethod
    def description(cls):
        """Returns a text string to illustrate the widget"""
        return """
<div> <!-- with widget id and class widget_class -->
  <div> <!-- normally hidden div, with class error_class -->
    <p> <!-- Any error text appears here --> </p>
  </div>
  <p>  <!-- with class para_class -->
  <!-- toptext appears in this paragraph, if there is no toptext the paragraph is not shown -->
  </p>
  <pre>
  <!-- environ variables are displayed here -->
  </pre>
</div>"""


class ShowCallData(Widget):
    """This class is a div, followed by a paragraph of toptext, and then
       a <pre> tag with text content showing the call_data dictionary.
       On error a paragraph above the toptext paragraph appears with error_class
    """

    error_location = (0,0,0)

    arg_descriptions = {'toptext':FieldArg("text", 'DEBUG MODE IS ON'),
                        'para_class':FieldArg("cssclass",""),
                        'error_class':FieldArg("cssclass", ""),
                        'show':FieldArg("boolean", True)  # set here as this value has a special textblock description
                       }

    def __init__(self, name=None, brief='', **field_args):
        """toptext: The text appearing above call_data
           error_class: If given the class of the error text"""
        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
        self[0] = tag.Part(tag_name="div", attribs={"style":"display:none;"})
        self[0][0] = tag.Part(tag_name="p")
        self[0][0][0] = ''
        self[1] = tag.Part(tag_name="p", hide_if_empty=True)
        self[2] = tag.Part(tag_name="pre")

    # special widget, not shown if debug is off

    def update(self, page, ident_list, environ, call_data, lang, ident_string, placename, embedded):
        """Sets self.show False if debug is False"""
        if not skiboot.get_debug():
            self.show = False
            return
        Widget.update(self, page, ident_list, environ, call_data, lang, ident_string, placename, embedded)

    def _build(self, page, ident_list, environ, call_data, lang):
        if self.get_field_value('error_class'):
            self[0].update_attribs({"class":self.get_field_value('error_class')})
        if self.error_status:
            self[0].del_one_attrib("style")
        if self.get_field_value('para_class'):
            self[1].update_attribs({"class":self.get_field_value('para_class')})
        self[1].text = self.get_field_value('toptext')
        self[2].text = pprint.pformat(call_data)


    @classmethod
    def description(cls):
        """Returns a text string to illustrate the widget"""
        return """
<div> <!-- with widget id and class widget_class -->
  <div> <!-- normally hidden div, with class error_class -->
    <p> <!-- Any error text appears here --> </p>
  </div>
  <p>  <!-- with class para_class -->
  <!-- toptext appears in this paragraph, if there is no toptext the paragraph is not shown -->
  </p>
  <pre>
  <!-- The call_data displayed here -->
  </pre>
</div>"""


class ShowResponders(Widget):
    """This class is a div containing a table with columns
       showing the responders called to access this page
    """

    error_location = (0,0,0)

    arg_descriptions = {'toptext':FieldArg("text", 'DEBUG MODE IS ON'),
                        'para_class':FieldArg("cssclass",""),
                        'error_class':FieldArg("cssclass", ""),
                        'paradiv_class':FieldArg("cssclass",""),
                        'table_class':FieldArg("cssclass",""),
                        'show':FieldArg("boolean", True)  # set here as this value has a special textblock description
                       }

    def __init__(self, name=None, brief='', **field_args):
        "Create the div and table, does not require any arguments"
        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
        self[0] = tag.Part(tag_name="div", attribs={"style":"display:none;"})
        self[0][0] = tag.Part(tag_name="p")
        self[0][0][0] = ''
        self[1] = tag.Part(tag_name="div")
        self[1][0] = tag.Part(tag_name="p", hide_if_empty=True)
        self[2] = tag.Part(tag_name="table")

    # special widget, not shown if debug is off

    def update(self, page, ident_list, environ, call_data, lang, ident_string, placename, embedded):
        """Sets self.show False if debug is False"""
        if not skiboot.get_debug():
            self.show = False
            return
        Widget.update(self, page, ident_list, environ, call_data, lang, ident_string, placename, embedded)

    def _build(self, page, ident_list, environ, call_data, lang):
        "Update the table to show responder data"
        if self.get_field_value('error_class'):
            self[0].update_attribs({"class":self.get_field_value('error_class')})
        if self.error_status:
            self[0].del_one_attrib("style")
        if self.get_field_value('paradiv_class'):
            self[1].update_attribs({"class":self.get_field_value('paradiv_class')})
        if self.get_field_value('para_class'):
            self[1][0].update_attribs({"class":self.get_field_value('para_class')})
        self[1][0].text = self.get_field_value('toptext')
        if self.get_field_value('table_class'):
            self[2].attribs = {"class":self.get_field_value('table_class')}
        # The ident_list is the list of responder idents used when calling this page
        if not ident_list:
            self[2][0] = tag.Part(tag_name="tr")
            self[2][0][0] = tag.Part(tag_name="td", text=page.ident.to_comma_str())
            self[2][0][1] = tag.Part(tag_name="td", text='This page')
            self[2][0][2] = tag.Part(tag_name="td", text=page.brief)
            return
        for index, ident in enumerate(ident_list):
            item = skiboot.from_ident(ident)
            self[2][index] = tag.Part(tag_name="tr")
            self[2][index][0] = tag.Part(tag_name="td", text=ident.to_comma_str())
            self[2][index][1] = tag.Part(tag_name="td", text=item.responder.__class__.__name__)
            self[2][index][2] = tag.Part(tag_name="td", text=item.brief)
        last_line = tag.Part(tag_name="tr")
        last_line[0] = tag.Part(tag_name="td", text=page.ident.to_comma_str())
        last_line[1] = tag.Part(tag_name="td", text='This page')
        last_line[2] = tag.Part(tag_name="td", text=page.brief)
        self[2].append(last_line)


    @classmethod
    def description(cls):
        """Returns a text string to illustrate the widget"""
        return """
<div> <!-- with widget id and class widget_class -->
  <div> <!-- normally hidden div, with class error_class -->
    <p> <!-- Any error text appears here --> </p>
  </div>
  <div>  <!-- with class paradiv_class -->
    <p>  <!-- with class para_class -->
    <!-- toptext appears in this paragraph, if there is no toptext the paragraph is not shown -->
    </p>
  </div>
  <table>   <!-- with class table_class -->
   <!-- a row for every Responder page visited, with columns; page ident, responder class, page brief --> 
  </table>
</div>"""


